SystemOrganization addCategory: #'Zinc-HTTP-Client-Server'!
SystemOrganization addCategory: #'Zinc-HTTP-Logging'!
SystemOrganization addCategory: #'Zinc-HTTP-Core'!
SystemOrganization addCategory: #'Zinc-HTTP-Exceptions'!
SystemOrganization addCategory: #'Zinc-HTTP-Streaming'!
SystemOrganization addCategory: #'Zinc-HTTP-Support'!

!String methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ ZnMimeType fromString: self! !

!String methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 1/4/2011 13:03'!
asZnUrl
	^ ZnUrl fromString: self! !

Stream subclass: #ZnBivalentWriteStream
	instanceVariableNames: 'stream binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnBivalentWriteStream commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:50' prior: 0!
I am ZnBivalentWriteStream, wrapping another write stream.

I allow both Characters and Strings as well as Integers (0 to: 255) and ByteArrays to be writtten to me.

The write stream that I wrap can be both binary as well as character based.

Part of Zinc HTTP Components.!

!ZnBivalentWriteStream classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/6/2010 12:49'!
on: writeStream
	"Wrap writeStream as a BivalentWriteStream, unless it is already one"
	
	(writeStream isKindOf: self) ifTrue: [ ^ writeStream ].
	
	^ self basicNew
		on: writeStream;
		yourself! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
close
	stream close! !

!ZnBivalentWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 20:54'!
isBinary
	binary isNil
		ifTrue: [ [ binary := stream isBinary ] on: MessageNotUnderstood do: [ :err | binary := false ] ].
	^ binary! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 20:46'!
next: count putAll: collection startingAt: offset
	self isBinary
		ifTrue: [ stream next: count putAll: collection asByteArray startingAt: offset ]
		ifFalse: [ stream next: count putAll: collection asString startingAt: offset ]! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPut: object
	self isBinary
		ifTrue: [ stream nextPut: object asInteger ]
		ifFalse: [ stream nextPut: object asCharacter ]! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPutAll: collection
	self isBinary
		ifTrue: [ stream nextPutAll: collection asByteArray ]
		ifFalse: [ stream nextPutAll: collection asString ]! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
on: writeStream
	stream := writeStream! !

!ZnBivalentWriteStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:08'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:12'!
space
	self nextPut: Character space! !

Stream subclass: #ZnBufferedWriteStream
	instanceVariableNames: 'stream buffer position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnBufferedWriteStream commentStamp: 'SvenVanCaekenberghe 12/8/2010 10:10' prior: 0!
I am ZnBufferedWriteStream.
I wrap a write stream and add buffering.

Make sure to always send me #flush or #close when you're done,
otherwise the last buffer might not yet have been written.

Part of Zinc HTTP Components.!

!ZnBufferedWriteStream classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/7/2010 21:20'!
on: writeStream
	^ self basicNew
		on: writeStream;
		yourself! !

!ZnBufferedWriteStream classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/9/2010 10:19'!
on: writeStream do: block
	"Execute block with as argument a ZnBufferedWriteStream on writeStream,
	making sure #flush is called at the end."
	
	| bufferedWriteStream |
	bufferedWriteStream := self on: writeStream.
	block value: bufferedWriteStream.
	bufferedWriteStream flush! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/8/2010 10:15'!
buffer
	buffer isNil 
		ifTrue: [ self sizeBuffer: self defaultBufferSize ].
	^ buffer! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 23:42'!
bufferFreeSize
	^ self bufferSize - position! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2010 10:08'!
bufferSize
	^ buffer isNil 
		ifTrue: [ self defaultBufferSize ]
		ifFalse: [ buffer size ]! !

!ZnBufferedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 21:22'!
close
	self flushBuffer.
	stream close! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:24'!
defaultBufferSize
	^ 4096! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:23'!
flush
	self flushBuffer.
	stream flush! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/7/2010 23:48'!
flushBuffer
	position = 0 ifTrue: [ ^ self ].
	position = self bufferSize
		ifTrue: [
			stream nextPutAll: buffer ]
		ifFalse: [
			(stream respondsTo: #next:putAll:startingAt:)
				ifTrue: [ stream next: position putAll: buffer startingAt: 1 ]
				ifFalse: [ stream nextPutAll: (buffer copyFrom: 1 to: position) ] ].
	position := 0! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/7/2010 23:39'!
flushBufferIfFull
	position = self bufferSize 
		ifTrue: [ self flushBuffer ]
! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2010 10:16'!
nextPut: object
	self flushBufferIfFull.
	position := position + 1.
	self buffer at: position put: object! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 20:44'!
nextPutAll: collection
	self flushBufferIfFull.
	collection size <= self bufferFreeSize
		ifTrue: [
			self buffer replaceFrom: position + 1 to: position + collection size with: collection.
			position := position + collection size ]
		ifFalse: [
			self flushBuffer.
			collection size > (self bufferSize / 2)
				ifTrue: [ stream nextPutAll: collection ]
				ifFalse: [ self nextPutAll: collection ] ]
! !

!ZnBufferedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 21:30'!
on: writeStream
	stream := writeStream.
	position := 0! !

!ZnBufferedWriteStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:19'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:28'!
sizeBuffer: size
	buffer := (stream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: size! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:20'!
space
	self nextPut: Character space! !

Stream subclass: #ZnChunkedReadStream
	instanceVariableNames: 'stream chunk position atEnd totalSize extraHeaders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnChunkedReadStream commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:01' prior: 0!
I am ZnChunkedReadStream, implementing HTTP 1.1 chunked transfer encoding on a wrapped streams.

Clients should read me until I am atEnd.

After I am completely read, I can tell you my totalSize and optional extraHeaders.

Part of Zinc HTTP Components.!

!ZnChunkedReadStream classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 14:10'!
new
	^ super new initialize! !

!ZnChunkedReadStream classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 19:43'!
on: stream
	^ self basicNew
		initialize;
		on: stream;
		yourself! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
atEnd
	self ensureChunkOrAtEnd.
	^ atEnd! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 08:11'!
binary
	stream binary! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
collectionSpecies
	^ self isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]
	! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 19:56'!
ensureChunkOrAtEnd
	atEnd 
		ifTrue: [ ^ self ].
	(chunk isNil or: [ position >= chunk size ]) 
		ifTrue: [ self getNextChunk ]
! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:21'!
extraHeaders
	^ extraHeaders! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 17:08'!
getExtraHeaders
	extraHeaders := ZnHeaders readFrom: stream! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 19:51'!
getNextChunk
	| size |
	(size := self getNextChunkSize) = 0
		ifTrue: [
			atEnd := true.
			chunk := position := nil.
			self getExtraHeaders ]
		ifFalse: [
			totalSize := totalSize + size.
			chunk := self collectionSpecies new: size.
			stream next: size into: chunk.
			stream next: 2. "consume CRLF"
			position := 0 ]! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 19:26'!
getNextChunkSize
	| line |
	(line := (ZnLineReader on: stream) nextLine) isEmpty ifTrue: [ ^ 0 ].
	^ Integer readFrom: (line copyUpTo: $;) base: 16! !

!ZnChunkedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 14:11'!
initialize
	atEnd := false.
	totalSize := 0! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
isBinary
	^ stream isBinary
! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
next
	self ensureChunkOrAtEnd.
	self atEnd ifTrue: [ ^ nil ].
	^ chunk at: (position := position + 1)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 15:27'!
next: anInteger 
	"We use our own collectionSpecies."

	^ self collectionSpecies streamContents: [ :out | | count |
		count := 0.
		[ self atEnd or: [ count >= anInteger ] ] whileFalse: [
			out nextPut: self next.
			count := count + 1 ] ]! !

!ZnChunkedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 14:13'!
on: readStream
	stream := readStream! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 15:30'!
position
	^ self atEnd 
		ifTrue: [ totalSize ]
		ifFalse: [ totalSize - chunk size + position ]! !

!ZnChunkedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
reset
	"We don't allow it"! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:10'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:20'!
totalSize
	^ totalSize! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 08:12'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	| newStream element |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd or: [ (element := self next) = anObject ] ]
		whileFalse: [ newStream nextPut: element ].
	^ newStream contents! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:10'!
upToEnd
	"We use out own collectionSpecies."

	| newStream |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd ] whileFalse: [ newStream nextPut: self next ].
	^ newStream contents! !

Stream subclass: #ZnLimitedReadStream
	instanceVariableNames: 'stream position limit binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnLimitedReadStream commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:02' prior: 0!
I am ZnLimitedReadStream, wrapping another read stream delegating to it 
but limiting reading to a fixed number of elements.

I can be atEnd while my wrapped stream is not.

My contentSpecies can be forced to be ByteArray (binary) or ByteString (ascii).

I do byte/char conversions on the fly (support bivalent access).

This implementation is really a kludge.

Part of Zinc HTTP Components.!

!ZnLimitedReadStream classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 11:15'!
on: stream limit: limit
	^ self basicNew
		on: stream limit: limit;
		yourself! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:38'!
ascii
	binary := false! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:07'!
atEnd
	^ position >= limit! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:36'!
binary
	binary := true! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:34'!
collectionSpecies
	"I can be set to binary or ascii explicitely.
	If not set, delegate to our wrapped stream."
	
	^ binary notNil
		ifTrue: [ 
			binary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ]
		ifFalse: [
			(stream respondsTo: #collectionSpecies)
				ifTrue: [ stream collectionSpecies ]  
				ifFalse: [ stream isBinary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:17'!
contentsOfEntireFile
	^ self upToEnd! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:06'!
isBinary
	^ binary isNil
		ifTrue: [ stream isBinary ]
		ifFalse: [ binary ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/21/2011 23:19'!
next
	"We do byte/char conversions on the fly."
	
	^ self atEnd 
		ifTrue: [ nil ] 
		ifFalse: [ 
			position := position + 1. 
			binary isNil
				ifTrue: [ stream next ] 
				ifFalse: [ 
					binary
					 	ifTrue: [ stream next asInteger ]
						ifFalse: [ stream next asCharacter ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/21/2011 23:19'!
next: requestedCount 
	"We use our own collectionSpecies."

	| newArray actualCount |
	actualCount := requestedCount min: (limit - position).
	newArray := self collectionSpecies new: actualCount.
	1 to: actualCount do: [ :index | newArray at: index put: self next ].
	^ newArray! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/21/2011 23:19'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	| target actualCount |
	(actualCount := requestedCount min: (limit - position)) = requestedCount
		ifTrue: [ target := collection ]
		ifFalse: [ target := collection species new: actualCount ].
	1 to: actualCount do: [ :index | target at: index put: self next ].
	^ target! !

!ZnLimitedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 13:06'!
on: readStream limit: integer
	stream := readStream.
	limit := integer.
	position := 0! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:09'!
position
	^ position! !

!ZnLimitedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/15/2010 12:50'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:20'!
reset
	"We don't allow it"! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:03'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:45'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	| newStream element |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd or: [ (element := self next) = anObject ] ]
		whileFalse: [ newStream nextPut: element ].
	^newStream contents! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:45'!
upToEnd
	"We use out own collectionSpecies."

	| newStream |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd ] whileFalse: [ newStream nextPut: self next ].
	^ newStream contents! !

Announcement subclass: #ZnLogEvent
	instanceVariableNames: 'timeStamp processId category message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnLogEvent commentStamp: 'SvenVanCaekenberghe 3/29/2011 13:27' prior: 0!
I am ZnLogEvent, an Announcement sent by an HTTP server or client containing logging information.

A log event has a TimeStamp, a Process ID, a category and a message.
The following categories are used: #info (I), #debug (D) and #transaction (T).  

Part of Zinc HTTP Components.!

!ZnLogEvent classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/28/2011 22:07'!
category: symbol message: string
	^ self new
		category: symbol;
		message: string;
		yourself    ! !

!ZnLogEvent classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/28/2011 22:08'!
debug: message
	^ self category: #debug message: message    ! !

!ZnLogEvent classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/28/2011 22:08'!
info: message
	^ self category: #info message: message    ! !

!ZnLogEvent classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/29/2011 09:26'!
transaction: message
	^ self category: #transaction message: message    ! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 22:05'!
category
	^ category! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:51'!
category: symbol
	category := symbol! !

!ZnLogEvent methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 12:51'!
initialize 
	self timeStamp: TimeStamp now.
	self category: #info.  
	self processId: ZnUtils currentProcessID     ! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 22:05'!
message
	^ message! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
message: string
	message := string! !

!ZnLogEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:51'!
printOn: stream
	self timeStamp printYMDOn: stream.
	stream space.
	self timeStamp printHMSOn: stream.
	stream space.
	self processId printOn: stream base: 10 length: 6 padded: true.
	stream space; nextPut: self category first asUppercase.
	self message isNil ifFalse: [ stream space; nextPutAll: self message ]! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:48'!
processId 
	^ processId ! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:48'!
processId: integer
	processId := integer ! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 22:05'!
timeStamp
	^ timeStamp! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:49'!
timeStamp: aTimeStamp
	timeStamp := aTimeStamp! !

Dictionary subclass: #ZnMultiValueDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnMultiValueDictionary commentStamp: 'SvenVanCaekenberghe 1/12/2011 11:13' prior: 0!
I am ZnMultiValueDictionary. 
I am a Dictionary.

I offer an #at:add: method to transparently create Array valued multi entries when needed.
My #keysAndValuesDo: is overwritten to transparently deal with multi entries.

Note that most other methods will show the actual value.

Part of Zinc HTTP Components.!

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:19'!
at: key add: value
	"Store value under key. If key already exists, store multiple values as Array"
	
	| existingValue newValue |
	existingValue := self at: key ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [
			existingValue isArray 
				ifTrue: [ existingValue copyWith: value ]
				ifFalse: [ Array with: existingValue with: value ] ].
	^ self at: key put: newValue 

! !

!ZnMultiValueDictionary methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 1/12/2011 11:10'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	super keysAndValuesDo: [ :key :value |
		value isArray
			ifTrue: [ 
				value do: [ :each | 
					block value: key value: each ] ]
			ifFalse: [
				block value: key value: value ] ]! !

Error subclass: #ZnParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnParseError commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:13' prior: 0!
ZnParseError is the superclass of all Zinc HTTP parsing related errors.

Part of Zinc HTTP Components.!

ZnParseError subclass: #ZnLineTooLong
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnLineTooLong commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:15' prior: 0!
ZnLineTooLong is signalled when an HTTP CRLF terminated input line exceeds a predefined limit.

Part of Zinc HTTP Components. !

!ZnLineTooLong classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
limit: limit
	^ self new 
		limit: limit; 
		yourself! !

!ZnLineTooLong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:32'!
limit
	^ limit! !

!ZnLineTooLong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:32'!
limit: anObject
	limit := anObject! !

ZnParseError subclass: #ZnPercentEncodingWrong
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnPercentEncodingWrong commentStamp: 'SvenVanCaekenberghe 2/28/2011 15:50' prior: 0!
ZnPercentEncodingWrong is signalled when a wrong percent encoding is encountered.

Part of Zinc HTTP Components.!

!ZnPercentEncodingWrong classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 15:50'!
string: string
	^ self new
		string: string;
		yourself ! !

!ZnPercentEncodingWrong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:49'!
string
	^ string! !

!ZnPercentEncodingWrong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:49'!
string: anObject
	string := anObject! !

ZnParseError subclass: #ZnPortNotANumber
	instanceVariableNames: 'port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnPortNotANumber commentStamp: 'SvenVanCaekenberghe 2/28/2011 15:23' prior: 0!
ZnPortNotANumber is signalled when an URL is parsed containing an illegal port specification.

Part of Zinc HTTP Components.!

!ZnPortNotANumber classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: port
	^ self new
		port: port;
		yourself! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port
	^ port! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: anObject
	port := anObject! !

ZnParseError subclass: #ZnUnknowHttpMethod
	instanceVariableNames: 'method'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknowHttpMethod commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:30' prior: 0!
ZnUnknowHttpMethod is signalled when an unknown/unsupported HTTP method is found while parsing a request.

Part of Zinc HTTP Components. !

!ZnUnknowHttpMethod classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
method: method
	^ self new
		method: method;
		yourself! !

!ZnUnknowHttpMethod methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
method
	^ method! !

!ZnUnknowHttpMethod methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
method: anObject
	method := anObject! !

ZnParseError subclass: #ZnUnknowHttpStatusCode
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknowHttpStatusCode commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:31' prior: 0!
ZnUnknowHttpStatusCode is signalled when an unknown/unsupported HTTP status code is found.

Part of Zinc HTTP Components. !

!ZnUnknowHttpStatusCode classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
code: code
	^ self new
		code: code;
		yourself! !

!ZnUnknowHttpStatusCode methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
code
	^ code! !

!ZnUnknowHttpStatusCode methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
code: anObject
	code := anObject! !

ZnParseError subclass: #ZnUnknowHttpVersion
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknowHttpVersion commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:30' prior: 0!
ZnUnknowHttpVersion is signalled when an unknown/unsupported HTTP version is found.

Part of Zinc HTTP Components. !

!ZnUnknowHttpVersion classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:35'!
version: version
	^ self new
		version: version;
		yourself! !

!ZnUnknowHttpVersion methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
version
	^ version! !

!ZnUnknowHttpVersion methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
version: anObject
	version := anObject! !

ZnParseError subclass: #ZnUnknowScheme
	instanceVariableNames: 'scheme'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknowScheme commentStamp: 'SvenVanCaekenberghe 3/18/2011 13:18' prior: 0!
ZnUnknowScheme is signalled when an unknown/unsupported URL scheme is found.

Part of Zinc HTTP Components. !

!ZnUnknowScheme classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/18/2011 13:17'!
scheme: scheme
	^ self new
		scheme: scheme;
		yourself! !

!ZnUnknowScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme
	^ scheme! !

!ZnUnknowScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme: anObject
	scheme := anObject! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 1/4/2011 13:30'!
asZnUrl
	^ZnUrl new
		scheme: self scheme;
		host: self authority;
		port: self port;
		segments: (self path collect: [ :each | each isEmpty ifTrue: [ #/ ] ifFalse: [ each ] ]);
		query: (self query isNil ifTrue: [ nil ] ifFalse: [ ZnUtils parseQueryFrom: self query readStream ]);
		fragment: self fragment;
		yourself! !

!MIMEType methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 1/4/2011 19:54'!
asZnMimeType
	^ ZnMimeType main: self main sub: self
! !

Object subclass: #ZnBasicAuthenticator
	instanceVariableNames: 'credentials realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicAuthenticator commentStamp: 'SvenVanCaekenberghe 9/30/2010 15:37' prior: 0!
I am ZnBasicAuthenticator.
I help servers handle HTTP Basic Authentication.
I implement the required #authenticateRequest:do:

I have a credentials dictionary with username->password associations.

Part of Zinc HTTP Components.!

!ZnBasicAuthenticator classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:50'!
defaultRealm
	^'ZincHTTPComponents'! !

!ZnBasicAuthenticator classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
new
	^ super new 
		initialize;
		yourself! !

!ZnBasicAuthenticator classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
username: username password: password
	^ self new
		atUsername: username putPassword: password;
		yourself! !

!ZnBasicAuthenticator classSide methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:55'!
username: username password: password realm: realm
	^ self new
		atUsername: username putPassword: password;
		realm: realm;
		yourself! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 17:19'!
atUsername: username putPassword: password
	credentials at: username put: password! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: ''!
authHeader
	^'Basic realm=', self realm ! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: 'MattKennedy 9/30/2010 15:56'!
authenticateRequest: request do: block
	^ (self isRequestAuthenticated: request)
		ifTrue: [ block value ]
		ifFalse: [ ZnResponse unauthorized: self authHeader ]! !

!ZnBasicAuthenticator methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/30/2010 15:37'!
initialize
	credentials := Dictionary new! !

!ZnBasicAuthenticator methodsFor: 'testing' stamp: ''!
isRequestAuthenticated: request
	| authorization username password |
	authorization := (request headers at: 'Authorization' ifAbsent: [ ^ false ]) findTokens: ' '.
	(authorization size = 2 and: [ authorization first = 'Basic' ]) ifFalse: [ ^ false ].
	authorization := (ZnUtils decodeBase64: authorization second) findTokens: ':'.
	authorization size = 2 ifFalse: [ ^ false ].
	username := authorization first.
	password := authorization second.
	^ (credentials at: username ifAbsent: [ nil ]) = password
	! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:53'!
realm
	^ realm ifNil: [realm := self class defaultRealm]! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:51'!
realm: anObject
	realm := anObject! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeAll
	credentials removeAll! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeUsername: username
	credentials removeKey: username! !

ZnBasicAuthenticator subclass: #ZnDigestAuthenticator
	instanceVariableNames: 'nonces'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestAuthenticator commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:47' prior: 0!
I am ZnDigestAuthenticator.
I help servers handle HTTP Digest Authentication.

I have a nonces dictionary with nonce->opaque associations. 
I generate a new nonce value for every 401 response I trigger.

Part of Zinc HTTP Components.!

!ZnDigestAuthenticator classSide methodsFor: 'testing' stamp: 'MattKennedy 10/1/2010 17:36'!
hasMD5Support
	^(self md5Hash: 'test') isNil not! !

!ZnDigestAuthenticator classSide methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 17:36'!
md5Hash: aString
	| hash |
	"Answer hash of aString as lowercase 32 digit hex String."
	Smalltalk at: #MD5 ifPresent: [:cls |
		^((cls new initialize) hashStream: (aString asByteArray readStream)) hex].
	hash := Smalltalk at: #CMD5Hasher ifPresent: [:cls |
		cls hashMessage: aString].
	hash ifNil: [
		hash := Smalltalk at: #TCryptoRandom ifPresent: [:cls |
			(cls basicNew md5HashMessage: aString) asInteger]].
	hash ifNotNil: [
		hash := hash hex asLowercase.
		(hash beginsWith: '16r') ifTrue: [hash := hash allButFirst: 3].
		hash := hash padded: #left to: 32 with: $0].
	^hash
! !

!ZnDigestAuthenticator classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 16:59'!
parseAuthRequest: aHeader
	| data dict |
	dict := Dictionary new.
	data := aHeader copyFrom: 8 to: aHeader size.
	(data subStrings: ',') do: [:fragment | | tokens i key value|
		tokens := fragment withBlanksTrimmed.
		i := tokens indexOf: $=.
		(i > 0) ifTrue: [
			key := tokens copyFrom: 1 to: i - 1.
			value := tokens copyFrom: i + 1 to: tokens size.  
			dict at: key put: (value copyFrom: 2 to: value size - 1)]].
	^dict! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 9/30/2010 17:25'!
a1for: username
	| password |
	password := credentials at: username ifAbsent: [^nil].
	^self class md5Hash: username, ':', self realm, ':', password
	! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 9/30/2010 17:25'!
a2forUrl: uri method: method
	^self class md5Hash: method, ':', uri
	! !

!ZnDigestAuthenticator methodsFor: 'public' stamp: 'MattKennedy 9/30/2010 16:31'!
authHeader
	| nonce opaque |
	nonce := self createNonce.
	opaque := self createOpaque.
	self nonces at: nonce put: opaque.
	^'Digest realm="', self realm, '", nonce="', nonce, '", opaque="', opaque, '"'! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 17:18'!
createNonce
	| pt |
	pt := TimeStamp current asString, ':', Random new next asString.
	^self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 17:21'!
createOpaque
	| pt |
	pt := Random new next asString, ':', TimeStamp current asString.
	^self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'testing' stamp: 'MattKennedy 9/30/2010 17:25'!
isRequestAuthenticated: request
	| authorization response cresponse a1 a2 nonce opaque |
	authorization := self class parseAuthRequest: (request headers at: 'Authorization' ifAbsent: [ ^ false ]).
	nonce := authorization at: 'nonce' ifAbsent: [^false].
	opaque := self nonces at: nonce ifAbsent: [^false].
	a1 := self a1for: (authorization at: 'username' ifAbsent: [^false]).
	a1 ifNil: [^false].
	a2 := self a2forUrl: (authorization at: 'uri' ifAbsent: ['/']) method: request method.
	response := self class md5Hash: (a1, ':', nonce, ':', a2).
	"(opaque = (authorization at: 'opaque' ifAbsent: [^false])) ifFalse: [^false]."
	cresponse := authorization at: 'response' ifAbsent: [^false].
	^(response = cresponse)! !

!ZnDigestAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 16:16'!
nonces 
	^nonces ifNil: [nonces := Dictionary new]! !

Object subclass: #ZnCharacterEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnCharacterEncoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:52' prior: 0!
I am ZnCharacterEncoder, I encode and decode Character objects to and from a binary stream.
I am an abstract class with following protocol:

#nextFromStream:
#nextPut:toStream:
#encodedByteCountFor:

The first two are compatible with TextConverter and subclasses.

Part of Zinc HTTP Components.!

ZnCharacterEncoder subclass: #ZnByteEncoder
	instanceVariableNames: 'byteToUnicode unicodeToByte'
	classVariableNames: 'ByteTextConverters'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnByteEncoder commentStamp: 'SvenVanCaekenberghe 1/25/2011 11:02' prior: 0!
I am ZnByteEncoder, a concrete subclass of ZnCharacterEncoder.
I handle single byte encodings where byte values 0 to 127 map to ASCII
and 128 to 255 are a permutation to Unicode characters.

I am reusing the specifications from the subclasses of ByteTextEncoder.

Part of Zinc HTTP Components.!

!ZnByteEncoder classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 13:02'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ ByteTextConverters includesKey: string! !

!ZnByteEncoder classSide methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 1/25/2011 12:26'!
initialize
	self initializeByteTextConverters! !

!ZnByteEncoder classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/25/2011 12:28'!
initializeByteTextConverters
	(ByteTextConverters := Dictionary new)
		at: 'cp-1250' put: CP1250TextConverter;
		at: 'cp-1253' put: CP1253TextConverter;
		at: 'iso-8859-2' put: ISO88592TextConverter; 
		at: 'latin-2' put: ISO88592TextConverter; 
		at: 'latin2' put: ISO88592TextConverter; 
		at: 'iso-8859-7' put: ISO88597TextConverter! !

!ZnByteEncoder classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/25/2011 12:47'!
initializeEncoder: encoder using: byteTextConverterClass
	"We are reusing the tables specified, computed and cached in subclasses of ByteTextConverter"
	
	encoder byteToUnicode: byteTextConverterClass byteToUnicodeTable.
	encoder unicodeToByte: byteTextConverterClass unicodeToByteTable! !

!ZnByteEncoder classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 12:44'!
newForEncoding: string
	"Return a new character encoder object for an encoding described by string.
	Search for a subclass that handles it and delegate"
	
	| encoder byteTextConverterClass |
	encoder := self new.
	byteTextConverterClass := ByteTextConverters at: string.
	self initializeEncoder: encoder using: byteTextConverterClass.
	^ encoder! !

!ZnByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/25/2011 12:31'!
byteToUnicode: map
	byteToUnicode := map! !

!ZnByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 12:19'!
encodedByteCountFor: character
	^ 1! !

!ZnByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 13:22'!
nextFromStream: stream
	| byteValue |
	^ (byteValue := stream next) < 128
		ifTrue: [ Character codePoint: byteValue ]
		ifFalse: [ | ch |
			"In the standard Squeak table some strange leading char was added"
			ch := byteToUnicode at: byteValue - 127.
			ch asUnicodeChar ]! !

!ZnByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 13:13'!
nextPut: character toStream: stream
	| code |
	(code := character charCode) < 128
		ifTrue: [ stream nextPut: code ]
		ifFalse: [ | ch |
			"In the standard Squeak table the byte values were converted to characters"
			ch := unicodeToByte at: code ifAbsent: [ self error: 'Character code outside encoder range' ].
			stream nextPut: ch charCode ]! !

!ZnByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/25/2011 12:31'!
unicodeToByte: map
	unicodeToByte := map! !

!ZnCharacterEncoder classSide methodsFor: 'accessing' stamp: ''!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	self subclassResponsibility  ! !

!ZnCharacterEncoder classSide methodsFor: 'instance creation' stamp: ''!
newForEncoding: string
	"Return a new character encoder object for an encoding described by string.
	Search for a subclass that handles it and delegate (subclassResponsibility).
	We default to ZnNullEncoder."
	
	| concreteSubclass encoding |
	encoding := string asLowercase.
	concreteSubclass := self subclasses 
		detect: [ :each | each handlesEncoding: encoding ] 
		ifNone: [ ZnNullEncoder ].
	^ concreteSubclass newForEncoding: encoding! !

!ZnCharacterEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:56'!
= anObject
	^ self class == anObject class! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: ''!
encodedByteCountFor: character
	"Return how many bytes are needed to encode character"
	
	self subclassResponsibility  ! !

!ZnCharacterEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:19'!
hash
	^ self class name hash! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: ''!
nextFromStream: stream
	"Read and return the next character from stream"
	
	self subclassResponsibility  ! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: ''!
nextPut: character toStream: stream
	"Write the encoding for character to stream"
	
	self subclassResponsibility ! !

ZnCharacterEncoder subclass: #ZnNullEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnNullEncoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:53' prior: 0!
I am ZnNullEncoder, a concrete subclass of ZnCharacterEncoder.
I perform no encoding or decoding at all for all characters with a code value below 256.
I can only be used for ASCII and Latin1 (ISO-8859-1).

Part of Zinc HTTP Components.!

!ZnNullEncoder classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 11:11'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ #( 'ascii' 'latin1' 'latin-1' 'iso-8859-1' ) includes: string! !

!ZnNullEncoder classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 11:19'!
newForEncoding: string
	"No further parametrization needed"
	
	^ self new! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:17'!
encodedByteCountFor: character
	^ 1! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 20:52'!
nextFromStream: stream
	^ Character codePoint: stream next! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 12:36'!
nextPut: character toStream: stream
	| code |
	(code := character charCode) < 256
		ifTrue: [ stream nextPut: code ]
		ifFalse: [ self error: 'Character code outside encoder range' ]! !

ZnCharacterEncoder subclass: #ZnUTF8Encoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnUTF8Encoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:55' prior: 0!
I am ZnUTF8Encoder, a concrete subclass of ZnCharacterEncoder.
I implement the variable length UTF-8 encoding and decoding of Unicode according to RFC 3629.

Part of Zinc HTTP Components.!

!ZnUTF8Encoder classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 11:11'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ #( 'utf-8' 'utf8' ) includes: string! !

!ZnUTF8Encoder classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 11:19'!
newForEncoding: string
	"No further parametrization needed"
	
	^ self new! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:23'!
encodedByteCountFor: character
	| code |
	code := character charCode.
	code < 128 ifTrue: [ ^ 1 ].
	code < 2048 ifTrue: [ ^ 2 ].
	code < 65535 ifTrue: [ ^ 3 ].
	code <= self maximumUTF8Code ifTrue: [ ^ 4 ].
	self error: 'Unicode character code point outside range'
! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/29/2010 21:19'!
maximumUTF8Code
	^ 16r10FFFF! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 20:52'!
nextFromStream: stream
	| code byte next |
	next := [ | nextByte |
		((nextByte := stream next) bitAnd: 2r11000000) = 2r10000000
			ifTrue: [ code := (code bitShift: 6) + (nextByte bitAnd: 2r00111111) ]
			ifFalse: [ self error: 'illegal UTF-8 encoding' ] ].
	(byte := stream next) < 128 ifTrue: [
		^ Character codePoint: byte ].
	(byte bitAnd: 2r11100000) = 2r11000000 ifTrue: [
		code := byte bitAnd: 2r00011111.
		next value.
		^ Character codePoint: code ].
	(byte bitAnd: 2r11110000) = 2r11100000 ifTrue: [
		code := byte bitAnd: 2r00001111.
		next value; value.
		^ Character codePoint: code ].
	(byte bitAnd: 2r11111000) = 2r11110000 ifTrue: [
		code := byte bitAnd: 2r00000111.
		next value; value; value.
		^ Character codePoint: code ].
	self error: 'illegal UTF-8 encoding'! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/30/2010 13:19'!
nextPut: character toStream: stream
	| code next |
	code := character charCode.
	next := [ :shift |
		stream nextPut: (2r10000000 + ((code bitShift: shift) bitAnd: 2r111111)) ].
	code < 128 ifTrue: [ 
		stream nextPut: code.
		^ character ].
	code < 2048 ifTrue: [ 
		stream nextPut: (2r11000000 + (code bitShift: -6)).
		next value: 0.
		^ character ].
	code < 65535 ifTrue: [ 
		stream nextPut: (2r11100000 + (code bitShift: -12)).
		next value: -6; value: 0.
		^ character ].
	code <= self maximumUTF8Code ifTrue: [ 
		stream nextPut: (2r11110000 + (code bitShift: -18)).
		next value: -12; value: -6; value: 0.
		^ character ].
	self error: 'Unicode character code point outside range'
! !

Object subclass: #ZnClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnClient commentStamp: 'SvenVanCaekenberghe 12/7/2010 15:18' prior: 0!
I am ZnClient.
I implement the client part of the HTTP protocol. 

My class side implements a number of convenience methods to do common HTTP client operations.

ZnClient get: 'http://caretaker.wolf359.be/small.html'.

We only handle one shot requests.
Have a look at ZnFixedCient and ZnHttpClient for alternatives with more options.

Part of Zinc HTTP Components.!

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:44'!
delete: urlObject
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	request := ZnRequest delete: url.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:44'!
delete: urlObject username: username password: password
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest delete: url)
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/6/2010 22:21'!
execute: request on: stream
	request writeOn: stream.
	stream flush.
	^ ZnResponse readFrom: stream! !

!ZnClient classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 20:59'!
executeOneShot: request on: stream
	request setConnectionClose.
	^ [ self execute: request on: stream ] ensure: [ stream close ]! !

!ZnClient classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/15/2010 19:20'!
executeStreaming: request on: stream
	request writeOn: stream.
	stream flush.
	^ ZnResponse readStreamingFrom: stream! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
get: urlObject
	"ZnClient get: 'http://caretaker.wolf359.be/small.html'."
	
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	request := ZnRequest get: url.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:44'!
get: urlObject username: username password: password
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/13/2010 16:05'!
getGif: urlObject
	"self getGif: 'http://homepage.mac.com/svc/ADayAtTheBeach/calculator.gif'."
	
	^ self 
		getImageOfType: ZnMimeType imageGif 
		usingParser: GIFReadWriter 
		fromUrl: urlObject 
	! !

!ZnClient classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
getImageOfType: mimeType usingParser: parserClass fromUrl: urlObject
	| url stream request response |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setAccept: mimeType.
	response := self executeOneShot: request on: stream.
	response status = 200 ifFalse: [
		^ self error: 'Expected OK response' ].
	response contentType = mimeType ifFalse: [
		^ self error: 'Expected a response of mime type ', mimeType printString ].
	^ (parserClass on: response entity readStream) nextImage
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/13/2010 16:04'!
getJpeg: urlObject
	"ZnClient getJpeg: 'http://caretaker.wolf359.be/sun-fire-x2100.jpg'."
	
	^ self 
		getImageOfType: ZnMimeType imageJpeg 
		usingParser: JPEGReadWriter 
		fromUrl: urlObject 
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/13/2010 16:06'!
getPng: urlObject
	"self getPng: 'http://www.pharo-project.org/images/pharo.png'."
	
	^ self 
		getImageOfType: ZnMimeType imagePng 
		usingParser: PNGReadWriter 
		fromUrl: urlObject 
	! !

!ZnClient classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
getStreamingImageOfType: mimeType usingParser: parserClass fromUrl: urlObject
	| url stream request response |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setConnectionClose; 
		setAccept: mimeType.
	response := nil.
	^ [ 
	response := self executeStreaming: request on: stream.
	response status = 200 ifFalse: [
		^ self error: 'Expected OK response' ].
	response contentType = mimeType ifFalse: [
		^ self error: 'Expected a JPEG' ].
	"Actually, the image parsers all just take the whole stream in memory, so the effort is mostly lost"
	(parserClass on: response entity stream) nextImage ]
	ensure: [ response notNil ifTrue: [ response entity consumeContent. stream close ] ]
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
head: urlObject
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest head: url)
		setConnectionClose.
	^ [
		request writeOn: stream.
		stream flush.
		ZnResponse readHeaderFrom: stream ] 
	ensure: [ stream close ]	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
head: urlObject username: username password: password
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest head: url)
		setBasicAuthenticationUsername: username password: password;
		setConnectionClose.
	^ [
		request writeOn: stream.
		stream flush.
		ZnResponse readHeaderFrom: stream ] 
	ensure: [ stream close ]		! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
post: urlObject data: entity
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest post: url)
		entity: entity.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
post: urlObject data: entity username: username password: password
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest post: url)
		entity: entity;
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:45'!
put: urlObject data: entity
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest put: url)
		entity: entity.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient classSide methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/7/2011 19:44'!
put: urlObject data: entity username: username password: password
	| url stream request |
	url := urlObject asZnUrl.
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest put: url)
		entity: entity;
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

Object subclass: #ZnConstants
	instanceVariableNames: ''
	classVariableNames: 'HTTPStatusCodes'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnConstants commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:01' prior: 0!
I am ZnConstants, holding various constants that we use.

Part of Zinc HTTP Components.!

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
defaultHTTPVersion
	^ 'HTTP/1.1'! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/31/2011 08:54'!
defaultServerString 
	^ self frameworkNameAndVersion ! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:49'!
defaultUserAgent
	^ self frameworkNameAndVersion ! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/10/2011 11:54'!
faviconBytes
	^ #(0 0 1 0 1 0 16 16 16 0 0 0 0 0 40 1 0 0 22 0 0 0 40 0 0 0 16 0 0 0 32 0 0 0 1 0 4 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 255 255 0 0 189 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 17 17 17 1 16 1 16 1 17 17 17 1 16 1 16 1 16 0 0 1 16 1 16 0 17 0 0 1 16 1 16 0 1 16 0 1 16 1 16 0 0 17 0 1 17 17 16 0 0 1 16 1 1 17 0 0 0 0 17 0 0 0 0 1 17 17 17 0 0 0 0 1 17 17 17 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0) asByteArray! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:36'!
frameworkLicense
	"http://en.wikipedia.org/wiki/MIT_License"
	
	^ 'MIT License'! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 17:12'!
frameworkMCVersion
	MCWorkingCopy managersForClass: self class do: [ :each | ^ each ancestors first name ]! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:35'!
frameworkName
	^ 'Zinc HTTP Components'! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:47'!
frameworkNameAndVersion
	^ self frameworkName, ' ', self frameworkVersion ! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/31/2011 08:58'!
frameworkVersion
	^ '1.0'! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:53'!
httpStatusCodes
	^ HTTPStatusCodes ! !

!ZnConstants classSide methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/6/2010 15:51'!
initialize 
	self initializeHTTPStatusCodes! !

!ZnConstants classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/6/2010 16:22'!
initializeHTTPStatusCodes 
	"This is the official list from http://www.iana.org/assignments/http-status-codes"

	HTTPStatusCodes := Dictionary newFromPairs: #(
		100 'Continue'
		101 'Switching Protocols'
		102 'Processing'
		200 'OK'
		201 'Created'
		202 'Accepted'
		203 'Non-Authoritative Information'
		204 'No Content'
		205 'Reset Content'
		206 'Partial Content'
		207 'Multi-Status'
		208 'Already Reported'
		226 'IM Used'
		300 'Multiple Choices'
		301 'Moved Permanently'
		302 'Found'
		303 'See Other'
		304 'Not Modified'
		305 'Use Proxy'
		306 'Reserved'
		307 'Temporary Redirect'
		400 'Bad Request'
		401 'Unauthorized'
		402 'Payment Required'
		403 'Forbidden'
		404 'Not Found'
		405 'Method Not Allowed'
		406 'Not Acceptable'
		407 'Proxy Authentication Required'
		408 'Request Timeout'
		409 'Conflict'
		410 'Gone'
		411 'Length Required'
		412 'Precondition Failed'
		413 'Request Entity Too Large'
		414 'Request-URI Too Long'
		415 'Unsupported Media Type'
		416 'Requested Range Not Satisfiable'
		417 'Expectation Failed'
		422 'Unprocessable Entity'
		423 'Locked'
		424 'Failed Dependency'
		425 'Reserved for WebDAV'
		426 'Upgrade Required'
		500 'Internal Server Error'
		501 'Not Implemented'
		502 'Bad Gateway'
		503 'Service Unavailable'
		504 'Gateway Timeout'
		505 'HTTP Version Not Supported'
		506 'Variant Also Negotiates (Experimental)'
		507 'Insufficient Storage'
		508 'Loop Detected'
		510 'Not Extended')! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2011 23:31'!
knownHTTPMethods
	^ #(GET PUT POST DELETE HEAD TRACE OPTIONS CONNECT PATCH)! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:22'!
knownHTTPVersions
	^ #('HTTP/1.1' 'HTTP/1.0')! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:19'!
remoteAddressHeader
	^ 'X-Zinc-Remote-Address'! !

!ZnConstants classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 13:40'!
welcomePageHtml
	^ '<html>
<head>
<title>Zinc HTTP Components</title>
<style type="text/css">
body {
  color: black;
  background: white;
  width: 900px;
  font-family: Verdana, Arial, Helvetica, sans-serif;
  font-size: 13px;
}
p {
  width: 600px;
  padding: 0 20px 10px 20px;
} 
ul,ol {
  width: 600px;
  padding: 0 5px 5px 30px;
} 
#logo {
  color: orange;
  font-family: Helvetica, sans-serif;
  font-weight: bold;
  font-size: 100px;
}
</style>
</head>
<body>
<div id="logo">Zn</div>
<h1>Zinc HTTP Components</h1>
<p>
Welcome to Zinc HTTP Components, 
a modern, open-source Smalltalk framework to deal with the HTTP networking protocol.
</p>
<ul>
<li>Project homepage <a href="http://homepage.mac.com/svc/Zinc-HTTP-Components/">http://homepage.mac.com/svc/Zinc-HTTP-Components/</a></li> 
<li>SqueakSouce page <a href="http://www.squeaksource.com/ZincHTTPComponents.html">http://www.squeaksource.com/ZincHTTPComponents.html</a></li>
</ul>
<ul>
<li>Locally available pages <a href="/help">/help</a></li>
</ul>
<h4>May the Source be with you!!</h4>
</body>
</html>
'! !

Object subclass: #ZnCookie
	instanceVariableNames: 'attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookie commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent an HTTP Cookie token.

Part of Zinc HTTP Components.!

!ZnCookie classSide methodsFor: 'instance creation' stamp: 'MattKennedy 9/28/2010 13:20'!
fromString: aString
	^self new fromString: aString; yourself! !

!ZnCookie classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 15:00'!
fromString: aString for: urlObject
	| cookie |
	"Ensures a domain value if none provided by the cookie"
	cookie := self new fromString: aString; yourself.
	cookie domain ifNil: [ cookie domain: urlObject host ].
	^cookie! !

!ZnCookie methodsFor: 'testing' stamp: 'MattKennedy 9/28/2010 10:15'!
= aCookie
	(aCookie isKindOf: self class) ifFalse: [^false].
	^(aCookie name = self name) & (aCookie path = self path) & (aCookie domain = self domain)
	! !

!ZnCookie methodsFor: 'converting' stamp: 'MattKennedy 9/28/2010 13:37'!
asString
	| s |
	s := String new writeStream.
	self writeNameAndValueOn: s.
	^s contents! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:24'!
domain
	^attributes at: 'domain' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:25'!
domain: aDomain
	attributes at: 'domain' put: aDomain! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
expires
	^attributes at: 'expires' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
expires: anExpiration
	attributes at: 'expires' put: anExpiration! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 16:09'!
expiresTimeStamp
	| readStream |
	self expires ifNil: [ 
		^ TimeStamp current plusDays: 1 ].
	readStream := self expires readStream.
	[ readStream atEnd or: [ readStream peek isDigit ] ] 
		whileFalse: [ readStream next ].
	^ TimeStamp readFrom: readStream! !

!ZnCookie methodsFor: 'private' stamp: 'MattKennedy 9/27/2010 20:58'!
extraAttributes
	"Answer all attributes except name and value"
	^attributes associationsSelect: [:ea | (ea key = 'name' or: [ea key = 'value']) not]! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'MattKennedy 9/27/2010 20:38'!
fromString: aString
	| tokens val data i |
	tokens := aString subStrings: ';'.
	i := tokens first indexOf: $=.
	val := tokens first copyFrom: 1 to: i - 1.
	data := tokens first copyFrom: i + 1 to: tokens first size. 
	self name: val; value: (data withBlanksTrimmed).
	2 to: tokens size do: [ :idx | 
		self readAttribute: (tokens at: idx).].! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 21:05'!
httponly
	"This is not an RFC cookie attribute, but a supplement found in MS Internet Explorer and supported in PHP.
	 See http://msdn.microsoft.com/en-us/library/ms533046.aspx"
	^attributes at: 'httponly' ifAbsent: [false]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
httponly: aBoolean
	attributes at: 'httponly' put: aBoolean! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'MattKennedy 9/27/2010 20:21'!
initialize 
	attributes := Dictionary new.! !

!ZnCookie methodsFor: 'testing' stamp: 'MattKennedy 9/28/2010 10:31'!
isExpired
	^(self expiresTimeStamp asUTC < TimeStamp current asUTC) or: [self expires = 0]! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 14:57'!
isInDomain: urlObject
	| domain |
	domain := (self domain beginsWith: '.')
		ifTrue: [ self domain copyFrom: 2 to: self domain size ]
		ifFalse: [ self domain ].
	^urlObject host includesSubString: domain
	! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 14:57'!
isInPath: urlObject
	^urlObject pathPrintString includesSubString: self path
	! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
maxage
	^self expires! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:22'!
name
	^attributes at: 'name'! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
name: aName
	attributes at: 'name' put: aName! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:07'!
path
	^attributes at: 'path' ifAbsent: ['/']! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:08'!
path: aPath
	attributes at: 'path' put: aPath! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'MattKennedy 9/28/2010 13:25'!
readAttribute: aToken
	| i key data |
	i := aToken indexOf: $=.
	i > 0
		ifTrue: [
			key := (aToken copyFrom: 1 to: i - 1) withBlanksTrimmed asLowercase.
			data := (aToken copyFrom: i + 1 to: aToken size) withBlanksTrimmed.]
		ifFalse: [ 
			key := (aToken copyFrom: 1 to: aToken size) withBlanksTrimmed asLowercase.
			data := true].
		attributes at: key put: data.! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:27'!
secure
	^attributes at: 'secure' ifAbsent: [false]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
secure: aBoolean
	attributes at: 'secure' put: aBoolean! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:24'!
value
	^attributes at: 'value' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
value: aValue
	attributes at: 'value' put: aValue! !

!ZnCookie methodsFor: 'printing' stamp: 'MattKennedy 9/28/2010 13:36'!
writeNameAndValueOn: aStream
	aStream nextPutAll: self name, '=', self value! !

!ZnCookie methodsFor: 'printing' stamp: 'MattKennedy 9/27/2010 21:01'!
writeOn: aStream
	aStream nextPutAll: self name, '=', self value.
	self extraAttributes associationsDo: [:ea |
		aStream nextPutAll: '; ', ea key.
		(ea value isKindOf: Boolean) ifFalse: [aStream nextPutAll: '=', ea value]]! !

Object subclass: #ZnCookieJar
	instanceVariableNames: 'cookies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookieJar commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent a container for storing HTTP Cookies for a client session.

Part of Zinc HTTP Components.!

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:37'!
add: aCookie
	(self removeIfExpired: aCookie) ifTrue: [^nil].
	self cookies do: [:cookie|
		cookie = aCookie ifTrue: [ self cookies remove: cookie]].
	self cookies add: aCookie
	! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:12'!
cookieAt: aName forUrl: urlObject
	(self cookiesForUrl: urlObject) do: [:cookie | cookie name = aName ifTrue: [^cookie]].
	^nil! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:24'!
cookies 
	^cookies ifNil: [cookies := Bag new]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 13:33'!
cookiesForUrl: urlObject
	^self cookies select: [:cookie | 
		(cookie isExpired not) & (cookie isInDomain: urlObject) & (cookie isInPath: urlObject)]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 13:36'!
remove: aCookie
	self cookies remove: aCookie ifAbsent: [nil]! !

!ZnCookieJar methodsFor: 'operations' stamp: 'MattKennedy 9/28/2010 10:29'!
removeIfExpired: aCookie
	aCookie isExpired ifTrue: [ self remove: aCookie. ^true].
	^false! !

Object subclass: #ZnCredential
	instanceVariableNames: 'domain realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:56' prior: 0!
I am ZnCredential, an abstract class representing credentials in a domain and realm.

Part of Zinc HTTP Components.!

ZnCredential subclass: #ZnBasicCredential
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:58' prior: 0!
I am ZnBasicCredential, a concrete subclass of ZnCredential.
I implement basic HTTP authentication.
I hold a username and password.

Part of Zinc HTTP Components.!

!ZnBasicCredential classSide methodsFor: 'accessing' stamp: ''!
authName
	^'Basic' ! !

!ZnBasicCredential methodsFor: 'accessing' stamp: ''!
authTokenForUrl: urlObject method: method
	^'Basic ', (ZnUtils encodeBase64: (username, ':', password))  ! !

!ZnBasicCredential methodsFor: 'testing' stamp: ''!
isComplete 
	^username notNil & password notNil ! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password
	^ password! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password: anObject
	password := anObject! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username
	^ username! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username: anObject
	username := anObject! !

ZnBasicCredential subclass: #ZnDigestCredential
	instanceVariableNames: 'nonce opaque'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:59' prior: 0!
I am ZnDigestCredential, a concrete subclass of ZnCredential and subclass of ZnBasicCredential.
I implement digest HTTP authentication.
I hold a nounce and opaque.

Part of Zinc HTTP Components.!

!ZnDigestCredential classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:29'!
authName
	^'Digest'! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'MattKennedy 9/30/2010 17:26'!
a1
	^ZnDigestAuthenticator md5Hash: self username, ':', self realm, ':', self password! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'MattKennedy 9/30/2010 17:26'!
a2ForUrl: urlObject method: method
	"This doesn't support all qop options yet."
	^ZnDigestAuthenticator md5Hash: method, ':', (self uriPathFrom: urlObject)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 17:02'!
authTokenForUrl: urlObject method: method
	| result |
	result := String new writeStream.
	result 
		nextPutAll: 'Digest username="', self username, '", ';
		nextPutAll: 'nonce="', self nonce, '", ';
		nextPutAll: 'realm="', self realm, '", ';
		nextPutAll: 'uri="', (self uriPathFrom: urlObject ), '", ';
		nextPutAll: 'response="', (self responseForUrl: urlObject method: method), '", '.
	self opaque isNil ifFalse: [
		result nextPutAll: 'opaque="', self opaque, '"'].
	^result contents! !

!ZnDigestCredential methodsFor: 'testing' stamp: 'MattKennedy 10/6/2010 17:17'!
isComplete 
	^super isComplete & nonce notNil ! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:25'!
linkToAuthRequest: aHeader
	| data |
	data := self parseAuthRequest: aHeader.
	self nonce: (data at: 'nonce').
	self realm: (data at: 'realm' ifAbsent: [''])! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce
	^ nonce! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce: anObject
	nonce := anObject! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque
	^ opaque! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque: anObject
	opaque := anObject! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/30/2010 16:33'!
parseAuthRequest: aHeader
	^ZnDigestAuthenticator parseAuthRequest: aHeader! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'MattKennedy 9/30/2010 16:40'!
responseForUrl: urlObject method: method
	^ZnDigestAuthenticator md5Hash: self a1, ':', self nonce, ':', (self a2ForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:28'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: (self authTokenForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 14:23'!
uriPathFrom: urlObject
	^urlObject pathPrintString ifNil: ['/']! !

!ZnCredential classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:28'!
authName
	self subclassResponsibility ! !

!ZnCredential classSide methodsFor: 'accessing class hierarchy' stamp: 'MattKennedy 9/29/2010 16:19'!
findHandlerFor: anAuthHeader
	| type i |
	i := anAuthHeader indexOf: Character space.
	type := anAuthHeader copyFrom: 1 to: i - 1.
	self allSubclassesDo: [:subclass |
		subclass authName = type ifTrue: [^subclass]].
	^nil! !

!ZnCredential classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 20:49'!
findRealmFor: anAuthHeader
	(anAuthHeader subStrings: ',') do: [:token |
		"String>>trimBoth: is a grease method, so not sure if we should be using it."
		(token includesSubString: 'realm=') 
			ifTrue: [ ^((token subStrings: '=') second)]].
	^nil! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 16:14'!
authToken
	^self authTokenForUrl: nil method: nil ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:48'!
authTokenForUrl: urlObject method: method
	self subclassResponsibility ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain
	^ domain! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain: anObject
	domain := anObject! !

!ZnCredential methodsFor: 'testing' stamp: 'MattKennedy 10/6/2010 17:16'!
isComplete
	^self subclassResponsibility ! !

!ZnCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:10'!
linkToAuthRequest: aHeader
	"Link this credential to the WWW-Authenticate request in the provided header. 
	Used for authentication methods which require parameters provided by the server
	(e.g. Digest authentication)."
	realm := self class findRealmFor: aHeader! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 21:45'!
realm
	^ realm ifNil: [realm := '']! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
realm: anObject
	realm := anObject! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:42'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: self authToken! !

Object subclass: #ZnDefaultServerDelegate
	instanceVariableNames: 'prefixMap byteArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnDefaultServerDelegate commentStamp: 'SvenVanCaekenberghe 3/31/2011 10:14' prior: 0!
I am ZnDefaultServerDelegate.
I function as a delegate for ZnServer, implementing #handleRequest:

I implement responses to the following prefixes:

/echo - an echo text of request information for debugging purposes
/dw-bench - a dynamic html page for benchmarkig purposes
/unicode - a unicode test page
/random - a random string (/random/32 for a specific size)
/bytes - bytes according to a pattern (/bytes/32 for a specific size)
/favicon.ico - a Zn favicon
/status - a server status page
/help - lists all page prefixes
/ - an html welcome page

Without any matches, I respond with a page not found.

Part of Zinc HTTP Components.!

!ZnDefaultServerDelegate classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/10/2010 15:17'!
new
	^ super new initialize! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 1/4/2011 14:52'!
bytes: request
	"Answer a number of bytes according to a pattern as an application/octet-stream.
	Optionally take the size from the URI, as in /bytes/32 for 32 bytes, default to 64 bytes.
	The main goal is to be efficient, esp. in benchmarks, that's why we cache (non-thread-safe)"
	
	| count |
	count := Integer readFrom: request uri lastPathSegment ifFail: [ 64 ].
	(byteArray isNil or: [ byteArray size ~= count ]) 
		ifTrue: [
			byteArray := ByteArray new: count.
			1 to: count - 1 do: [ :each | 
				byteArray at: each put: (#(65 66 67 68) at: (each \\ 4 + 1)) ].
			byteArray at: count put: 10].
	^ ZnResponse ok: (ZnEntity bytes: byteArray)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:30'!
dwbench: request
	"Reply with the dynamic DW-Bench HTML page"

	^ ZnResponse ok: (ZnEntity html: self generateDWBench)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/10/2010 15:16'!
echoRequest: request	
	"Echo request by generating a plain text response useful for debugging purposes"
	
	| info entity |
	info := self generateEchoRequestString: request.
	entity := ZnEntity text: info.
	^ ZnResponse ok: entity! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:39'!
favicon: request
	"Answer a nice favicon for browsers to display"

	| entity |
	entity := ZnEntity with: ZnConstants faviconBytes type: 'image/vnd.microsoft.icon'. 
	^ ZnResponse ok: entity! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/31/2011 09:37'!
generateDWBench
	"Generate the dynamic DW-Bench HTML page"
	
	^ String streamContents: [ :stream |
		stream nextPutAll: '<html><head><title>DW-Bench Dynamic</title></head>'.
		stream nextPutAll: '<body><h1>DW-Bench Dynamic</h1><table border=1>'.
		1 to: 25 do: [ :row |
			stream nextPutAll: '<tr>'.
			1 to: 25 do: [ :col |
				stream nextPutAll: '<td>'; print: row * col; nextPutAll: '</td>' ].
			stream nextPutAll: '</tr>' ].
		stream nextPutAll: '</table><p>'; print: TimeStamp now; nextPutAll: '</p></body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 13:55'!
generateEchoRequestString: request
	"Generate a string echoing the request"
	
	^ String streamContents: [ :stream |
		stream nextPutAll: 'Hello World from Zinc HTTP Components !!'; crlf.
		stream nextPutAll: request method; nextPutAll: ' request for '.
		request uri printPathQueryFragmentOn: stream.
		stream crlf; nextPutAll: 'with headers'; crlf.
		request headersDo: [ :key :value |
			stream space; nextPutAll: key; nextPutAll: ': '; nextPutAll: value; crlf ].
		request entity isNil ifFalse: [
			stream nextPutAll: ' containing '; print: request entity ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/28/2011 22:09'!
generateHelp
	"Generate an HTML page with links to all pages I support"
	
	| title |
	title := self class name, ' Help'.
	^ String streamContents: [ :stream |
		stream << '<html><head><title>'; << title; << '</title></head>'.
		stream << '<body><h1>'; << title; << '</h1><h3>Available Pages</h3><ul>'.
		prefixMap keys do: [ :each | 
			stream << '<li><a href="'; << each; << '">'; << each; << '</a></li>' ].
		stream << '<ul></body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/31/2011 10:09'!
generateStatus
	"Generate an HTML page with the state of the server"
	
	^ String streamContents: [ :stream |
		stream nextPutAll: '<html><head><title>ZnServer Status</title></head>'.
		stream nextPutAll: '<body><h1>ZnServer Status</h1>'.
		stream nextPutAll: '<pre>'; nextPutAll: SmalltalkImage current vmStatisticsReportString; nextPutAll: '</pre>'.  
		stream nextPutAll: '<h1>Processes</h1><ol>'.
		self processes do: [ :each |
			stream 
				nextPutAll: '<li>'; 
				nextPutAll: each name; space; nextPut: $[; print: each priority; nextPut: $]; nextPutAll: '<br/>'; nextPutAll: each printString;
				nextPutAll: '</li>' ].
		stream nextPutAll: '</ol>'.  
		stream nextPutAll: '</body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/31/2011 09:37'!
generateUnicodeTest
	"Generate an HTML page containing all Unicode characters with code points between 0 and 16r024F in a nice table"
	
	^ String streamContents: [ :stream |
		stream nextPutAll: '<html><head><title>Unicode Test Page</title></head>'.
		stream nextPutAll: '<body><h1>Unicode Test Page</h1><table>'.
		0 to: 16r024F by: 8 do: [ :row |
			stream nextPutAll: '<tr>'.
			row to: row + 7 do: [ :each |
				stream nextPutAll: '<td>U+'.
				stream nextPutAll: (each printPaddedWith: $0 to: 4 base: 16). 
				stream nextPut: $=; nextPut: each asCharacter.
				stream nextPutAll: '</td>' ].
			stream nextPutAll: '</tr>' ].
		stream nextPutAll: '</table></body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 4/28/2011 22:13'!
handleRequest: request
	| prefix prefixHandler |
	prefix := request uri isSlash 
		ifTrue: [ prefixMap at: '/' ifAbsent: [ nil ] ] 
		ifFalse: [ request uri firstPathSegment ].
	prefixHandler := prefixMap at: prefix ifAbsent: [ nil ].
	prefixHandler isNil
		ifFalse: [ 
			^ prefixHandler isSymbol 
				ifTrue: [ self perform: prefixHandler with: request ]
				ifFalse: [ prefixHandler value: request ] ].
	^ ZnResponse notFound: request uri! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 3/31/2011 09:17'!
help: request
	"Reply with a dynamic HTML page containing links to all pages I support"

	^ ZnResponse ok: (ZnEntity html: self generateHelp)
	! !

!ZnDefaultServerDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/28/2011 22:09'!
initialize
	(prefixMap := Dictionary new)
		at: 'welcome' put: #welcome:;
		at: 'help' put: #help:;
		at: 'echo' put: #echoRequest:;
		at: 'dw-bench' put: #dwbench:;
		at: 'favicon.ico' put: #favicon:;
		at: 'random' put: #random:;
		at: 'bytes' put: #bytes:;
		at: 'status' put: #status:;
		at: 'unicode' put: #unicode:;
		at: '/' put: 'welcome'! !

!ZnDefaultServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 13:28'!
prefixMap
	^ prefixMap ! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/31/2011 09:46'!
processes
	^ (Process allSubInstances
		reject: [ :each | each isTerminated or: [ each = Processor activeProcess ] ])
		sorted: [ :a :b | a priority >= b priority ] ! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 1/4/2011 14:52'!
random: request
	"Answer a text plain UTF-8 encoded string of random hex characters.
	Optionally take the size from the URI, as in /random/32 for 32 characters.
	Default to 64 characters with a newline at the end"
	
	| count hexString |
	count := Integer readFrom: request uri lastPathSegment ifFail: [ 64 ].
	hexString := String new: count streamContents: [ :stream |
		count - 1 timesRepeat: [ stream nextPut: '0123456789ABCDEF' atRandom ].
		stream lf ].
	^ ZnResponse ok: (ZnEntity text: hexString)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 3/31/2011 09:18'!
status: request
	"Reply with a dynamic HTML page describing the state of the server"

	^ ZnResponse ok: (ZnEntity html: self generateStatus)
	! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:40'!
unicode: request
	"Answer a Unicode Test HTML page"
	
	^ ZnResponse ok: (ZnEntity html: self generateUnicodeTest)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:39'!
welcome: request
	"Answer a nice HTML welcome page"
	
	| html entity |
	html := ZnConstants welcomePageHtml copy 
		replaceAll: Character cr with: Character lf.
	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

Object subclass: #ZnEntity
	instanceVariableNames: 'contentType contentLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:03' prior: 0!
I am ZnEntity, an abstract HTTP Entity.

An entity has a content type (mime-type), a content length (byte length) and a contents.

Entities can encode and decode themselves to and from binary streams.

Part of Zinc HTTP Components.!

ZnEntity subclass: #ZnApplicationFormUrlEncodedEntity
	instanceVariableNames: 'fields representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnApplicationFormUrlEncodedEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnApplicationFormUrlEncodedEntity, a concrete HTTP Entity 
holding a set of fields (key value pairs) that is form URL encoded.
I am a ZnEntity.

Part of Zinc HTTP Components.!

!ZnApplicationFormUrlEncodedEntity classSide methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:41'!
designatedMimeType
	^ ZnMimeType applicationFormUrlEncoded ! !

!ZnApplicationFormUrlEncodedEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 22:23'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:23'!
addAll: keyedCollection
	keyedCollection keysAndValuesDo: [ :key :value |
		self at: key put: value ].
	^ keyedCollection ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:07'!
at: key
	^ self fields at: key! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:23'!
at: key add: value
	self invalidateRepresentation.
	^ self fields at: key add: value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:23'!
at: key ifAbsent: block
	self invalidateRepresentation.
	^ self fields at: key ifAbsent: block! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:23'!
at: key put: value
	self invalidateRepresentation.
	^ self fields at: key put: value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:19'!
computeRepresentation
	representation := String streamContents: [ :stream | self writeRepresentationOn: stream ].
	contentLength := representation size! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:22'!
contentLength
	contentLength isNil ifTrue: [ self computeRepresentation ].
	^ contentLength ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 16:19'!
contents
	^ self fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:00'!
fields
	^ fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 1/31/2011 12:16'!
fieldsDo: block
	self fields keysAndValuesDo: block! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/31/2011 12:15'!
initialize
	super initialize.
	fields := ZnMultiValueDictionary new.
	self contentType: self class designatedMimeType ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:17'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:41'!
isEmpty
	^ self fields isEmpty! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty ifFalse: [ 
		stream space.
		fields printElementsOn: stream ]! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 19:35'!
readFrom: stream
	"We parse from stream, limited to content lenght if available."
	
	| readStream |
	representation := nil.
	self contentLength isNil
		ifTrue: [ readStream := stream ]
		ifFalse: [ (readStream := ZnLimitedReadStream on: stream limit: self contentLength) ascii ].
	fields := ZnUtils parseQueryFrom: readStream! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:22'!
writeOn: stream
	"In most use cases, contentLength will have been asked first so that a representation is cached,
	there probably is no need to cache the representation generated here"
	
	representation isNil
		ifTrue: [ self writeRepresentationOn: stream ] 
		ifFalse: [ stream nextPutAll: representation ] ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/13/2010 21:32'!
writeRepresentationOn: stream
	ZnUtils writeQueryFields: self fields on: stream! !

ZnEntity subclass: #ZnByteArrayEntity
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnByteArrayEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnByteArrayEntity, a concrete HTTP Entity based on a ByteArray.
It is used to hold binary (non textual) data.
I am a ZnEntity.

Part of Zinc HTTP Components.!

!ZnByteArrayEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:13'!
bytes: byteArray
	^ (self type: ZnMimeType applicationOctetStream length: byteArray size)
		bytes: byteArray;
		yourself ! !

!ZnByteArrayEntity classSide methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 17:22'!
designatedMimeType
	"I have no designated mime type, I can handle all binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes
	^ bytes! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes: anObject
	bytes := anObject! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:05'!
contents
	^ self bytes! !

!ZnByteArrayEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:53'!
isEmpty 
	^ self bytes isNil or: [ self bytes isEmpty ]! !

!ZnByteArrayEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self bytes notNil ifTrue: [ 
		stream space; print: self bytes ]! !

!ZnByteArrayEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/20/2010 10:37'!
readFrom: stream
	self contentLength isNil
		ifTrue: [ 
			self bytes: stream upToEnd.
			self contentLength: self bytes size ]
		ifFalse: [ | byteArray |
			byteArray := ByteArray ofSize: self contentLength.
			stream next: self contentLength into: byteArray.
			self bytes: byteArray ]! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 19:27'!
readStream
	^ self bytes readStream! !

!ZnByteArrayEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
writeOn: stream
	stream nextPutAll: self bytes! !

!ZnEntity classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 13:57'!
byteArrayEntityClass
	^ ZnByteArrayEntity ! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:00'!
bytes: bytes
	^ self byteArrayEntityClass bytes: bytes! !

!ZnEntity classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
concreteSubclassForType: mimeType
	"Answer the concrete ZnEntity subclass that handles mimeType"
	
	^ self allSubclasses 
		detect: [ :each | 
			each designatedMimeType = mimeType ]
		ifNone: [ 
			mimeType isBinary 
				ifTrue: [ self byteArrayEntityClass ] 
				ifFalse: [ self stringEntityClass ] ]
! !

!ZnEntity classSide methodsFor: 'testing' stamp: ''!
designatedMimeType
	"Answer the mime type we are designated to handle.
	See #concreteSubclassForType: "
	
	self subclassResponsibility  ! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: ''!
html: html
	^ self stringEntityClass html: html! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 17:07'!
new
	^ super new initialize ! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: ''!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new buffered entity of mimeType and length from stream.
	The concrete subclass depends on mimeType"
	
	| newEntity |
	newEntity := (self concreteSubclassForType: mimeType) 
						type: mimeType length: length.
	newEntity readFrom: stream.
	^ newEntity 
! !

!ZnEntity classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2010 13:48'!
stringEntityClass
	^ ZnStringEntity! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: ''!
text: text
	^ self stringEntityClass text: text! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 22:16'!
type: type
	"Create a new entity with a given mime type and (currently) undetermined length"
	
	^ self new
		contentType: type;
		yourself! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:06'!
type: type length: length
	"Create a new entity with a given mime type and length"
	
	^ self new
		contentType: type;
		contentLength: length;
		yourself! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:02'!
with: object
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	You will probably want to set the mime type to something more specific"
	
	^ object isString 
		ifTrue: [ self text: object ] 
		ifFalse: [ self bytes: object ]! !

!ZnEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 20:39'!
with: object type: type
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	The entity is set to mime type"
	
	^ (self with: object)
		contentType: type;
		yourself! !

!ZnEntity methodsFor: 'accessing' stamp: ''!
contentLength 
	"Content length can be nil when it is (currently) unknown."
	
	^ contentLength ! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentLength: integer
	contentLength := integer! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentType
	^ contentType ! !

!ZnEntity methodsFor: 'accessing' stamp: ''!
contentType: object
	"We only allow assignment compatible with our designated mime type"
	| newType |
	newType := object asZnMimeType.
	contentType = newType 
		ifTrue: [ ^ self ]
		ifFalse: [ 
			(self class designatedMimeType isNil or: [ self class designatedMimeType = newType ])
				ifTrue: [ contentType := newType ] ]! !

!ZnEntity methodsFor: 'accessing' stamp: ''!
contents
	self subclassResponsibility! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:25'!
hasContentLength
	^ contentLength notNil! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:26'!
hasContentType
	^ contentType notNil! !

!ZnEntity methodsFor: 'initialize-release' stamp: ''!
initialize ! !

!ZnEntity methodsFor: 'testing' stamp: ''!
isEmpty 
	self subclassResponsibility  ! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:19'!
printContentTypeAndLengthOn: stream
	self hasContentType ifTrue: [ 
		stream print: self contentType; space ].
	self hasContentLength ifTrue: [
		stream print: self contentLength; nextPut: $B ]! !

!ZnEntity methodsFor: 'printing' stamp: ''!
printContentsOn: stream
	self printContentTypeAndLengthOn: stream! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:24'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self printContentsOn: stream.
	stream nextPut: $)! !

!ZnEntity methodsFor: 'initialize-release' stamp: ''!
readFrom: stream
	self subclassResponsibility  ! !

!ZnEntity methodsFor: 'writing' stamp: ''!
writeOn: stream
	self subclassResponsibility  ! !

ZnEntity subclass: #ZnMultiPartFormDataEntity
	instanceVariableNames: 'parts representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMultiPartFormDataEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnMultiPartFormDataEntity, a concrete HTTP Entity 
holding multi part form data that is encoded according to specific rules.
I am a ZnEntity.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMultiPartFormDataEntity classSide methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:42'!
designatedMimeType
	^ ZnMimeType multiPartFormData  ! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 10:29'!
addPart: mimePart
	self invalidateRepresentation.
	self parts add: mimePart! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 20:11'!
computeRepresentation
	representation := ByteArray streamContents: [ :stream | 
		self writeRepresentationOn: (ZnBivalentWriteStream on: stream) ].
	contentLength := representation size! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 17:11'!
contentLength
	contentLength isNil ifTrue: [ self computeRepresentation ].
	^ contentLength! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 19:58'!
contentType: object
	"We only allow assignment compatible with our designated mime type.
	The main/sub must be equal but the parameters must be allowed to be different"
	
	| newType |
	newType := object asZnMimeType.
	self class designatedMimeType = newType
		ifTrue: [ contentType := newType ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 19:14'!
contents
	^ self parts! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/31/2011 12:13'!
generateBoundary
	^ String streamContents: [ :stream | | letters |
			stream nextPutAll: 'Boundary-Zn-'.
			letters := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.
			8 timesRepeat: [ stream nextPut: letters atRandom ] ]
	! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:17'!
getBoundary
	"Return the multipart/form-data MIME part boundary.
	This should really be there, set externally or internally.
	See #initialize and #mimeTypeWithBoundary"
	
	^ (self contentType parameterAt: 'boundary') withoutQuoting ! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/5/2010 17:17'!
initialize
	super initialize.
	self contentType: self mimeTypeWithBoundary  
	! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 19:15'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnMultiPartFormDataEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:43'!
isEmpty
	^ parts isNil or: [ parts isEmpty ]! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:19'!
mimeTypeWithBoundary
	| mimeType |
	mimeType := self class designatedMimeType copy.
	mimeType parameterAt: 'boundary' put: self generateBoundary.
	^ mimeType! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'PaulDeBruicker 4/10/2011 11:55'!
parse: bytes boundary: boundary
	| next start fullBoundary |
	fullBoundary := #(45 45) asByteArray , boundary.
	start := fullBoundary size + 1.
	next := bytes
		indexOfSubCollection: fullBoundary
		startingAt: start 
		ifAbsent: [ bytes size ].
	[ next < (bytes size - 2) ] whileTrue: [
		self addPart: (ZnMimePart readFrom: (bytes copyFrom: start + 2 to: next - 3) readStream).
		start := next + fullBoundary size.
		next := bytes
			indexOfSubCollection: fullBoundary
			startingAt: start
			ifAbsent: [ bytes size ] ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:12'!
partNamed: fieldName
	^ self partNamed: fieldName ifNone: [ self error: 'Cannot find part named ', fieldName asString ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:18'!
partNamed: fieldName ifNone: block
	parts isNil ifTrue: [ block value ].
	^ self parts detect: [ :each | each fieldName = fieldName ] ifNone: block! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 20:07'!
parts
	parts isNil ifTrue: [ parts := OrderedCollection new ].
	^ parts! !

!ZnMultiPartFormDataEntity methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 2/27/2011 20:13'!
partsDo: block
	parts isNil ifTrue: [ ^ self ].
	parts do: block  ! !

!ZnMultiPartFormDataEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty ifFalse: [ 
		stream space.
		self parts printElementsOn: stream ]! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/27/2011 17:11'!
readFrom: stream
	"Switch to streaming implementation later on"
	
	| bytes |
	"We have to use the contentLength instance variable instead of the accessor else a representation will be computed"
	contentLength isNil
		ifTrue: [ 
			bytes := stream upToEnd ]
		ifFalse: [
			bytes := ByteArray ofSize: contentLength.
			stream next: contentLength into: bytes ].
	self parse: bytes boundary: self getBoundary asByteArray! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/5/2010 20:28'!
writeOn: stream
	"We always go via our representation because we need a bivalent stream"
	
	representation isNil ifTrue: [ self computeRepresentation ].
	stream nextPutAll: representation! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:24'!
writeRepresentationOn: stream
	| boundary |
	boundary := self getBoundary.
	self parts do: [ :each |
		stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: String crlf.
		each writeOn: stream.
		stream nextPutAll: String crlf ].
	stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: '--'; nextPutAll: String crlf
! !

ZnEntity subclass: #ZnStreamingEntity
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStreamingEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnStreamingEntity, a concrete HTTP Entity based on a stream.
I am a ZnEntity.

When a ZnStreamingEntity is instanciated using #readFrom: 
its content type and length are set but no body is actually read.
Instead, stream is set to a ZnLimitedReadStream that clients can/must
use to read from.

When the client is done, either all data must be read or #consumeContent must be called.
A client should normally not close the stream.

A ZnStreamingEntity can also be instanciated with an existing read stream.
During #writeOn: this stream is copied to the output stream.

Part of Zinc HTTP Components.!

!ZnStreamingEntity classSide methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:23'!
designatedMimeType
	"I have no designated mime type, I can handle all content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStreamingEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 17:13'!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new streaming entity of mimeType and length from stream.
	Clients can/must use the embedded ZnLimitedReadStream to read all content, 
	or call #consumeContent when they are done. They should not close the stream."
	
	^ (self type: mimeType length: length)
		readFrom: stream;
		yourself
! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:10'!
consumeContent
	"To be improved later on using some kind of larger buffer"
	
	[ self stream atEnd ] 
		whileFalse: [ self stream next ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/28/2010 13:39'!
contents
	"Since this is a non-repeatable entity, 
	#contents will #consumeContent, 
	this can thus only be done once"
	
	^ self stream upToEnd ! !

!ZnStreamingEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 18:25'!
isEmpty 
	"We actually don't know"
	
	^ false! !

!ZnStreamingEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 19:34'!
readFrom: readStream
	self contentLength isNil
		ifTrue: [ 
			self stream: readStream ]
		ifFalse: [ | limitedStream |
			limitedStream := ZnLimitedReadStream on: readStream limit: self contentLength.
			self contentType isBinary 
				ifTrue: [ limitedStream binary ] 
				ifFalse: [ limitedStream ascii ]. 
			self stream: limitedStream ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:14'!
readStream 
	^ self stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream 
	^ stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream: readStream
	stream := readStream! !

!ZnStreamingEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/17/2010 13:43'!
writeOn: outputStream
	ZnUtils 
		streamFrom: self stream 
		to: outputStream 
		size: self contentLength.
	stream close! !

ZnEntity subclass: #ZnStringEntity
	instanceVariableNames: 'string encoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStringEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:05' prior: 0!
I am ZnStringEntity, a concrete HTTP Entity based on a String.
It is used to hold textual (non-binary) data.
I am a ZnEntity.

Optionally, an encoding is used to convert to and from bytes.
The default encoding it UTF-8.

Part of Zinc HTTP Components.!

!ZnStringEntity classSide methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:58'!
designatedMimeType
	"I have no designated mime type, I can handle all non-binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStringEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
html: string
	^ (self type: ZnMimeType textHtml)
		string: string;
		yourself! !

!ZnStringEntity classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
text: string
	^ (self type: ZnMimeType textPlain)
		string: string;
		yourself! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/27/2011 10:27'!
computeContentLength
	| length |
	self string isNil ifTrue: [ ^ self ].
	self initializeEncoder.
	length := self string 
		inject: 0 
		into: [ :sum :each | sum + (encoder encodedByteCountFor: each) ].
	contentLength := length! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 10:26'!
contentLength
	contentLength isNil ifTrue: [ self computeContentLength ].
	^ contentLength! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
contents
	^ self string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:59'!
encoder
	self initializeEncoder.
	^ encoder! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
encoder: anEncoding
	(encoder notNil and: [ anEncoding ~= encoder ]) ifTrue: [ self invalidateContentLength ].
	encoder := anEncoding! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/14/2010 12:09'!
hasEncoder
	^ encoder notNil! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/25/2011 13:42'!
initializeEncoder
	| charSet newEncoder |
	self hasEncoder ifTrue: [ ^ self ].
	newEncoder := (charSet := contentType charSet) isNil
		ifTrue: [ ZnNullEncoder new ]
		ifFalse: [ ZnCharacterEncoder newForEncoding: charSet ].
	self encoder: newEncoder! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/2/2010 12:20'!
invalidateContentLength
	self contentLength: nil! !

!ZnStringEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:54'!
isEmpty 
	^ self string isNil or: [ self string isEmpty ] ! !

!ZnStringEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self string notNil ifTrue: [ 
		stream space; nextPutAll: self string ]! !

!ZnStringEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/1/2011 13:03'!
readFrom: stream
	self contentLength isNil 
		ifTrue: [ self readUpToEndFrom: stream ] 
		ifFalse: [ self readLimitedFrom: stream ]! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 12:24'!
readLimitedFrom: stream
	| readStream newString |
	readStream := ZnLimitedReadStream on: stream limit: self contentLength.
	self initializeEncoder.
	newString := String streamContents: [ :stringStream |
		[ readStream atEnd ] whileFalse: [
			stringStream nextPut: (encoder nextFromStream: readStream) ] ].
	self string: newString! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
readStream
	^ self string readStream! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 12:58'!
readUpToEndFrom: stream
	| newString |
	self initializeEncoder.
	newString := String streamContents: [ :stringStream |
		[ [ stream atEnd ] whileFalse: [
			stringStream nextPut: (encoder nextFromStream: stream) ] ] 
			on: ConnectionClosed 
			do: [ :exception | exception return ] ].
	self string: newString! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
string
	^ string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
string: aString
	(string notNil and: [ aString ~= string ]) ifTrue: [ self invalidateContentLength ].
	string := aString! !

!ZnStringEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/2/2010 13:09'!
writeOn: stream
	self initializeEncoder.
	self string do: [ :each | 
		encoder nextPut: each toStream: stream ]! !

Object subclass: #ZnEntityReader
	instanceVariableNames: 'headers stream streaming allowReadingUpToEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnEntityReader commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:59' prior: 0!
I am ZnEntityReader, I help reading ZnEntities from a stream given meta data in headers.

I deal with chunking and gzip decoding.

Part of Zinc HTTP Components.!

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowReadingUpToEnd
	allowReadingUpToEnd := true! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowsReadingUpToEnd
	^ allowReadingUpToEnd notNil and: [ allowReadingUpToEnd ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:39'!
canReadContent
	^ self hasContentLength or: [ self isChunked or: [ self allowsReadingUpToEnd ] ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:10'!
contentLength
	^ (self headers includesKey: 'Content-Length')
		ifTrue: [ self headers contentLength ]
		ifFalse: [ nil ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:11'!
contentType
	^ (self headers includesKey: 'Content-Type') 
		ifTrue: [ self headers contentType ] 
		ifFalse: [ ZnMimeType applicationOctetStream ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:35'!
hasContentLength
	^ self headers includesKey: 'Content-Length'! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
headers
	^ headers! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
headers: znHeaders
	headers := znHeaders! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 13:10'!
isChunked
	| transferEncoding |
	transferEncoding := self headers at: 'Transfer-Encoding' ifAbsent: [ ^ false ].
	^ transferEncoding beginsWith: 'chunked'! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:55'!
isGzipped
	| contentEncoding |
	contentEncoding := self headers at: 'Content-Encoding' ifAbsent: [ ^ false ].
	^ contentEncoding = 'gzip'! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 16:45'!
isIdentityEncoded
	^ self isGzipped not! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:03'!
isStreaming
	^ streaming notNil and: [ streaming ]! !

!ZnEntityReader methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/1/2011 17:50'!
readEntity
	| entity |
	self canReadContent ifFalse: [ ^ nil ].
	entity := self readEntityFromStream.
	^ entity isEmpty
		ifTrue: [ nil ]
		ifFalse: [ entity ]    ! !

!ZnEntityReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 17:50'!
readEntityFromStream
	| entity decodedEntityLength chunkedStream |
	self canReadContent ifFalse: [ ^ nil ].
	chunkedStream := self isChunked ifTrue: [ stream := ZnChunkedReadStream on: stream ] ifFalse: [ nil ].
	self isGzipped 
		ifTrue: [ 
			self hasContentLength ifTrue: [ stream := ZnLimitedReadStream on: stream limit: self contentLength ].
			decodedEntityLength := nil.
			stream := GZipReadStream on: stream ]
		ifFalse: [ decodedEntityLength := self contentLength ].
	entity := (self isStreaming ifTrue: [ ZnStreamingEntity ] ifFalse: [ ZnEntity ])
			readFrom: stream
			usingType: self contentType 
			andLength: decodedEntityLength.
	self isChunked ifTrue: [ 
		self isIdentityEncoded ifTrue: [ entity contentLength: stream totalSize ].
		self headers addAll: chunkedStream extraHeaders ].
	^ entity! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
stream
	^ stream! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
stream: readStream
	stream := readStream ! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:31'!
streaming
	streaming := true! !

Object subclass: #ZnFixedClient
	instanceVariableNames: 'host port connection username password lastUsed log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnFixedClient commentStamp: 'SvenVanCaekenberghe 12/7/2010 15:22' prior: 0!
I am ZnFixedClient.
I implement the client part of the HTTP protocol. 

I talk to one fixed host:port combination, reusing the same connection for each request.
If I lose the connection, I simply reconnect. 
If something goes wrong, I do one retry.
I optionally use basic authentication.

(ZnFixedClient host: 'caretaker.wolf359.be')
	get: 'index.html';
	get: 'small.html'

Part of Zinc HTTP Components.!

ZnFixedClient subclass: #ZnExtendedFixedClient
	instanceVariableNames: 'preProcessHook postProcessHook newConnectionHook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnExtendedFixedClient commentStamp: 'SvenVanCaekenberghe 3/29/2011 14:02' prior: 0!
I am ZnExtendedFixedClient, a ZnFixedClient with some hooks to extend its behavior.

Part of Zinc HTTP Components!

!ZnExtendedFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
newConnection
	| newConnection |
	newConnection := super newConnection.
	newConnectionHook notNil ifTrue: [
		newConnectionHook value: newConnection ].  
	^ newConnection ! !

!ZnExtendedFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:03'!
newConnectionHook: block
	"If not nil, block will be given each connection just after a new one gets created"
	
	newConnectionHook := block! !

!ZnExtendedFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:03'!
postProcessHook: block
	"If not nil, block will be given each response just before it is returned"
	
	postProcessHook := block! !

!ZnExtendedFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
postProcessResponse: response
	super postProcessResponse: response.  
	postProcessHook notNil 
		ifTrue: [ postProcessHook value: response ]! !

!ZnExtendedFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:03'!
preProcessHook: block
	"If not nil, block will be given each request just before it gets executed"
	
	preProcessHook := block! !

!ZnExtendedFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
preProcessRequest: request
	super preProcessRequest: request.  
	preProcessHook notNil ifTrue: [
		preProcessHook value: request ]  ! !

!ZnFixedClient classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 21:04'!
host: hostname
	^ self host: hostname port: 80! !

!ZnFixedClient classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 20:06'!
host: hostname port: port
	^ self new
		host: hostname;
		port: port;
		yourself! !

!ZnFixedClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/20/2011 12:10'!
close
	connection ifNil: [ ^ self ].
	self log debug: 'Closing connection'.
	connection close. 
	connection := nil.
	lastUsed := nil! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 15:10'!
connection
	(self isConnected and: [ self isConnectionReusable ])
		ifFalse: [ 
			self close.
			connection := self newConnection ].
	^ connection! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:05'!
delete: uri
	| request |
	request := ZnRequest delete: (self fixedUrl: uri).
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 4/20/2011 12:38'!
executeRequest: request
	"Execute request to our fixed scheme://host:port and return the response.
	Make sure that the URI inside request is as processed by #fixedUrl:"

	| initialMilliseconds response |
	initialMilliseconds := Time millisecondClockValue.
	self preProcessRequest: request.
	(response := self executeRequestOnceIgnoringErrors: request) isNil
		ifTrue: [
			response := [ self close; executeRequestOnce: request ] 
				ifCurtailed: [ self log debug: [ 'Failed' ]. self close ] ].
	self logRequest: request response: response started: initialMilliseconds.  
	^ response! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:23'!
executeRequestOnce: request
	| stream response |
	stream := self connection.
	request writeOn: stream.
	stream flush.
	response := ZnResponse readFrom: stream.
	self postProcessResponse: response.
	^ response! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:24'!
executeRequestOnceIgnoringErrors: request
	^ [ self executeRequestOnce: request ] 
		on: Error 
		do: [ :err | 
			self log debug: [ 'Once Ignoring ', err printString ].
			nil ]! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/12/2011 16:13'!
fixedUrl: uri
	"Accept either a string that we parse, or a ZnURL object,
	and make sure it points to the fixed scheme://host:port that we talk too.
	This is necessary because we try to reuse the connection."
	
	| url |
	url := uri asZnUrl.
	self host ifNil: [ self error: 'Host has not been set' ].
	url
		scheme: #http;
		host: self host;
		port: self port.
	^ url! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 20:41'!
get: uri
	| request |
	request := ZnRequest get: (self fixedUrl: uri).
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:06'!
host
	^ host! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:06'!
host: hostname
	host := hostname! !

!ZnFixedClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/28/2011 15:07'!
isConnected
	^ connection notNil and: [ 
		connection isConnected and: [ 
			connection isOtherEndConnected ] ]! !

!ZnFixedClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/28/2011 15:12'!
isConnectionReusable
	^ connection notNil and: [ 
		lastUsed notNil
			and: [ TimeStamp now <= (lastUsed + self maxKeepAlive) ] ]! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
log
	^ log ifNil: [ log := ZnLogSupport new ]! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:16'!
logRequest: request response: response started: initialMilliseconds
	self log transaction: [
		String streamContents: [ :stream |
			request requestLine printMethodAndUriOn: stream.  
			stream space; print: response code.
			response hasEntity ifTrue: [
				stream space; print: response entity contentLength; nextPut: $B ].  
			stream space; print: (Time millisecondsSince: initialMilliseconds); nextPutAll:  'ms' ] ]! !

!ZnFixedClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 15:40'!
logToTranscript
	self log addListener: ZnTranscriptLogger new    ! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/28/2011 15:09'!
maxKeepAlive
	"Maximum age of kept alive connections that we try to reuse"
	
	^ 5 seconds! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:32'!
newConnection
	self log debug: 'Opening connection'.
	^ ZnNetworkingUtils socketStreamToHostNamed: self host port: self port! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:11'!
port
	port isNil ifTrue: [ port := 80 ].
	^ port! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:07'!
port: integer
	port := integer! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:05'!
post: uri data: entity
	| request |
	(request := ZnRequest post: (self fixedUrl: uri))
		entity: entity.
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:04'!
postProcessResponse: response
	lastUsed := TimeStamp now.
	response isConnectionClose
		ifTrue: [ self close ]! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:04'!
preProcessRequest: request
	(username notNil and: [ password notNil ]) ifTrue: [
		request setBasicAuthenticationUsername: username password: password ]! !

!ZnFixedClient methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:53'!
printOn: stream
	super printOn: stream.
	self host notNil ifTrue: [
		stream nextPut: $(; nextPutAll: 'http://'; nextPutAll: self host; nextPut: $:; print: self port; nextPut: $) ]! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:06'!
put: uri data: entity
	| request |
	(request := ZnRequest put: (self fixedUrl: uri))
		entity: entity.
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 21:02'!
username: user password: pwd
	username := user.
	password := pwd! !

Object subclass: #ZnHTTPSocketFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnHTTPSocketFacade commentStamp: '<historical>' prior: 0!
I am ZnHTTPSocketFacade.

On my class side I implement all necessary replacement methods of HTTPSocket.

The interface (signature and semantics) of this API is defined by HTTPSocket.

Overwriting these HTTPSocket methods with mine will effectively redirect all
HTTP client access in this Smalltalk image through Zinc HTTP Components.

Part of Zinc HTTP Components.!

!ZnHTTPSocketFacade classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/7/2011 09:34'!
constructMultiPartFormDataEntity: arguments
	| entity |
	entity := ZnMultiPartFormDataEntity new.
	arguments keysAndValuesDo: [ :key :values |
		values do: [ :value |
			entity addPart: ((value isKindOf: MIMEDocument)
				ifFalse: [ 
					 ZnMimePart fieldName: key value: value ]
				ifTrue: [ | fileEntity contents |
					contents := value content isNil 
						ifTrue: [ (FileStream fileNamed: value url pathForFile) contentsOfEntireFile ] 
						ifFalse: [ value content ].
					fileEntity := ZnEntity with: contents type: value contentType.
					ZnMimePart fieldName: key fileName: value url pathForFile entity: fileEntity ]) ] ].
	^ entity
! !

!ZnHTTPSocketFacade classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/21/2010 08:12'!
execute: request on: stream
	"All our requests are one shot"
	
	request setConnectionClose.
	^ [ ZnClient execute: request on: stream ] ensure: [ stream close ]! !

!ZnHTTPSocketFacade classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/13/2010 21:47'!
extendHeaders: headers with: object 
	(object isNil or: [ object isEmpty ]) ifTrue: [ ^ headers ].
	object isString 
		ifTrue: [ headers addAll: (ZnHeaders readFrom: object readStream) ] 
		ifFalse: [ headers withAll: object ].
	^ headers
	
! !

!ZnHTTPSocketFacade classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/30/2011 13:57'!
extendUrl: urlObject withArguments: arguments
	| url |
	url := urlObject asZnUrl.
	(arguments isNil or: [ arguments isEmpty ]) ifTrue: [ ^ url ].
	^ arguments isString
		ifTrue: [ 
			(url printString, (arguments first = $? ifTrue: [ arguments ] ifFalse: [ '?', arguments ])) asZnUrl ] 
		ifFalse: [
			url == urlObject ifTrue: [ url := url copy ].   
			arguments keysAndValuesDo: [ :key :value | 
				value do: [ :each | url queryAt: key add: each ] ].
			^ url ]! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url
	^ self 
		httpGet: url 
		accept: '*/*'! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url accept: mimeType
	^ self 
		httpGet: url 
		args: nil 
		accept: '*/*'! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url args: queryArguments accept: mimeType
	^ self 
		httpGet: url 
		args: queryArguments 
		accept: mimeType 
		request: nil! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 1/7/2011 19:46'!
httpGet: urlObject args: queryArguments accept: mimeType request: extraHeaders
	| url stream request response |
	url := self extendUrl: urlObject withArguments: queryArguments. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setAccept: mimeType.
	self extendHeaders: request headers with: extraHeaders. 
	response := self execute: request on: stream.
	^ self streamOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 1/7/2011 19:46'!
httpGet: urlObject args: queryArguments user: username passwd: password
	| url stream request response |
	url := self extendUrl: urlObject withArguments: queryArguments. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setBasicAuthenticationUsername: username password: password.
	response := self execute: request on: stream.
	^ self streamOrErrorStringFrom: response uri: url! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGetDocument: url
	^ self 
		httpGetDocument: url 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGetDocument: url accept: mimeType
	^ self 
		httpGetDocument: url 
		args: nil 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:18'!
httpGetDocument: url args: queryArguments
	^ self 
		httpGetDocument: url 
		args: queryArguments 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:18'!
httpGetDocument: url args: queryArguments accept: mimeType
	^ self 
		httpGetDocument: url 
		args: queryArguments 
		accept: 'application/octet-stream' 
		request: nil! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 1/7/2011 19:46'!
httpGetDocument: urlObject args: queryArguments accept: mimeType request: extraHeaders
	| url stream request response |
	url := self extendUrl: urlObject withArguments: queryArguments. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setAccept: mimeType.
	self extendHeaders: request headers with: extraHeaders. 
	response := self execute: request on: stream.
	^ self mimeDocumentOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/13/2010 16:07'!
httpGif: url
	^ ZnClient getGif: url! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/13/2010 16:08'!
httpJpeg: url
	^ ZnClient getJpeg: url! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/13/2010 16:08'!
httpPng: url
	^ ZnClient getPng: url! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 1/7/2011 19:46'!
httpPost: urlObject args: arguments accept: mimeType
	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest post: url) 
		setAccept: mimeType;
		entity: (ZnApplicationFormUrlEncodedEntity withAll: arguments).
	response := self execute: request on: stream.
	^ self streamOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 1/7/2011 19:46'!
httpPost: urlObject args: arguments user: username passwd: password
	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest post: url) 
		setBasicAuthenticationUsername: username password: password;
		entity: (ZnApplicationFormUrlEncodedEntity withAll: arguments).
	response := self execute: request on: stream.
	^ self mimeDocumentOrErrorStringFrom: response uri: url

! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/17/2010 20:05'!
httpPostDocument: url args: arguments
	^ self 
		httpPostDocument: url 
		args: arguments 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/17/2010 20:05'!
httpPostDocument: url args: arguments accept: mimeType
	^ self 
		httpPostDocument: url 
		args: arguments 
		accept: mimeType 
		request: nil! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 1/7/2011 19:46'!
httpPostDocument: urlObject args: arguments accept: mimeType request: extraHeaders
	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest post: url) 
		setAccept: mimeType;
		entity: (ZnApplicationFormUrlEncodedEntity withAll: arguments).
	self extendHeaders: request headers with: extraHeaders. 
	response := self execute: request on: stream.
	^ self mimeDocumentOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 2/7/2011 09:26'!
httpPostMultipart: urlObject args: arguments accept: mimeType request: extraHeaders
	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest post: url) 
		setAccept: mimeType;
		entity: (self constructMultiPartFormDataEntity: arguments).
	self extendHeaders: request headers with: extraHeaders. 
	response := self execute: request on: stream.
	^ self mimeDocumentOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade classSide methodsFor: 'operations put' stamp: 'SvenVanCaekenberghe 1/7/2011 19:46'!
httpPut: string to: urlObject user: username passwd: password 
	"We're implementing the weird semantics of the original ;-)"

	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnNetworkingUtils socketStreamToUrl: url.
	(request := ZnRequest put: url) 
		setBasicAuthenticationUsername: username password: password; 
		entity: (ZnEntity with: string).
	request headers removeKey: 'Content-Type'.
	response := self execute: request on: stream.
	^ String streamContents: [ :stringStream | response writeOn: stringStream ]! !

!ZnHTTPSocketFacade classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/21/2010 16:14'!
mimeDocumentOrErrorStringFrom: response uri: uri
	^ (#(200 201) includes: response statusLine code)
		ifTrue: [
			MIMEDocument
				contentType: response contentType asString
				content: response entity contents 
				url: uri ]
		ifFalse: [ 
			String streamContents: [ :stream |
				stream nextPutAll: 'HTTP request for '; print: uri; nextPutAll: ' failed with '.
				response statusLine writeOn: stream ] ]
		! !

!ZnHTTPSocketFacade classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 20:42'!
streamOrErrorStringFrom: response uri: uri
	^ (#(200 201) includes: response statusLine code)
		ifTrue: [
			(RWBinaryOrTextStream with: response entity contents) 
				reset; 
				yourself ]
		ifFalse: [ 
			String streamContents: [ :stream |
				stream nextPutAll: 'HTTP request for '; print: uri; nextPutAll: ' failed with '.
				response statusLine writeOn: stream ] ]
		! !

Object subclass: #ZnHeaders
	instanceVariableNames: 'headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHeaders commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:57' prior: 0!
I am ZnHeaders.
I am a collection of HTTP headers.
I can be used for generating and parsing.

Header names are normalized and used case insensitive.
Header values can be multi-valued.

Part of Zinc HTTP Components.!

!ZnHeaders classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 21:12'!
defaultRequestHeaders
	^ self new
		at: 'User-Agent' put: ZnConstants defaultUserAgent;
		at: 'Accept' put: '*/*';
		yourself! !

!ZnHeaders classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/31/2011 08:54'!
defaultResponseHeaders
	^ self new
		at: 'Server' put: ZnConstants defaultServerString;
		at: 'Date' put: ZnUtils httpDate;
		yourself! !

!ZnHeaders classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 17:07'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnHeaders classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 13:01'!
requestHeadersFor: url
	^ self defaultRequestHeaders
		at: 'Host' put: url authority;
		yourself! !

!ZnHeaders classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 21:45'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 15:05'!
acceptEntityDescription: entity
	"Take over the content type and length from entity"
	
	self
		at: 'Content-Type' put: entity contentType printString;
		at: 'Content-Length' put: entity contentLength printString! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:25'!
addAll: keyedCollection
	"Note that we use #at:add:"
	
	keyedCollection keysAndValuesDo: [ :key :value |
		self at: key add: value ].
	^ keyedCollection ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:25'!
at: headerName
	"Return the value stored under headerName,
	a String for single-valued headers, an Array of Strings for multi-valued headers"
	
	^ self headers at: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:48'!
at: headerName add: value
	"Store value under headerName, optionally turning it into a multi-valued header 
	when a value was already present"
	
	| normalizedKey existingValue newValue |
	normalizedKey := self normalizeHeaderKey: headerName.
	existingValue := self headers at: normalizedKey ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [
			existingValue isArray 
				ifTrue: [ existingValue copyWith: value ]
				ifFalse: [ Array with: existingValue with: value ] ].
	^ self headers at: normalizedKey put: newValue 
		
! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
at: headerName ifAbsent: block
	"Like #at: but executes block when the headerName is not found"
	
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers at: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:26'!
at: headerName put: value
	"Store value under headerName, replace existing entries"
	
	^ self headers at: (self normalizeHeaderKey: headerName) put: value! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:49'!
at: headerName put: value ifPresentMerge: binaryBlock
	"Store value under headerName, when there is an existing entry,
	stored the result of evaluating binaryBlock with old and new value"
	
	| normalizedKey existingValue newValue |
	normalizedKey := self normalizeHeaderKey: headerName.
	existingValue := self headers at: normalizedKey ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [ binaryBlock value: existingValue value: value ].
	^ self headers at: normalizedKey put: newValue 
		

! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:27'!
contentLength
	^ Integer 
		readFrom: (self headers at: 'Content-Length') 
		ifFail: [ self error: 'Illegal HTTP Content Length' ]! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:27'!
contentType
	^ ZnMimeType fromString: (self headers at: 'Content-Type')! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:12'!
hasContentLength
	^ self includesKey: 'Content-Length'! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:11'!
hasContentType
	^ self includesKey: 'Content-Type'! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/6/2010 17:03'!
headers
	headers isNil ifTrue: [ headers := Dictionary new ].
	^ headers! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:14'!
headersDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self isEmpty ifTrue: [ ^ self ].
	self headers keysAndValuesDo: [ :headerKey :headerValue |
		headerValue isArray
			ifTrue: [ 
				headerValue do: [ :each | 
					block value: headerKey value: each ] ]
			ifFalse: [
				block value: headerKey value: headerValue ] ]! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:13'!
includesKey: headerName
	^ self isEmpty not and: [ self headers includesKey: (self normalizeHeaderKey: headerName) ]
	! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:29'!
isDescribingEntity
	"Do I include enough information to describe an entity (i.e. content length and type) ?"
	
	^ (self headers includesKey: 'Content-Type') 
		and: [ self headers includesKey: 'Content-Length' ]
! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/6/2010 17:09'!
isEmpty 
	^ headers isNil or: [ self headers isEmpty ]! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/26/2010 18:13'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self headersDo: block! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/10/2010 20:04'!
normalizeHeaderKey: string
	"Test string to see if it has proper header key capitalization,
	if true, return string, if not, return a properly capitalized copy"
	
	^ (ZnUtils isCapitalizedString: string)
		ifTrue: [ string ] 
		ifFalse: [ ZnUtils capitalizeString: string ] ! !

!ZnHeaders methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:20'!
printOn: stream
	super printOn: stream.
	self isEmpty ifFalse: [ self headers printElementsOn: stream ]
	! !

!ZnHeaders methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 19:26'!
readFrom: stream
	| line reader |
	reader := ZnLineReader on: stream.
	[ (line := reader nextLine) isEmpty ] whileFalse: [
		self readOneHeaderFrom: line readStream ] ! !

!ZnHeaders methodsFor: 'private' stamp: 'MattKennedy 9/28/2010 13:14'!
readOneHeaderFrom: stream
	| key value |
	key := stream upTo: $:.
	[ stream peek == Character space ] whileTrue: [ stream next ].
	value := stream upToEnd.
	self at: key add: value! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 13:36'!
removeKey: headerName	
	^ self headers removeKey: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
removeKey: headerName ifAbsent: block
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers removeKey: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:39'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	self headersDo: [ :key :value |
		writeStream nextPutAll: key; nextPut: $:; space; nextPutAll: value; nextPutAll: String crlf ]! !

Object subclass: #ZnLineReader
	instanceVariableNames: 'stream buffer position limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnLineReader commentStamp: 'SvenVanCaekenberghe 9/27/2010 17:31' prior: 0!
I am ZnLineReader and I help reading CRLF terminated lines from a stream.
I can work on ascii and binary streams doing conversions as necessary.
I limit lines to a maximum length.
I work with a reusable buffer.

Part of Zinc HTTP Components.!

!ZnLineReader classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/27/2010 17:15'!
on: stream
	^ self new
		on: stream;
		yourself! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/28/2011 14:36'!
growBuffer
	| newBuffer |
	buffer size >= limit 
		ifTrue: [
			(ZnLineTooLong limit: limit) signal ].
	newBuffer := String new: buffer size * 2.
	newBuffer replaceFrom: 1 to: buffer size with: buffer.
	buffer := newBuffer ! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 17:30'!
limit: size
	limit := size! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:05'!
nextLine
	self reset.
	[ self processNext isNil ] whileFalse.
	^ buffer copyFrom: 1 to: position ! !

!ZnLineReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 17:30'!
on: readStream
	stream := readStream.
	buffer := String new: 64.
	self limit: 4096; reset! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 19:04'!
processNext
	| item |
	stream atEnd ifTrue: [ ^ nil ].
	item := stream next.
	stream isBinary ifTrue: [ item := item asCharacter ].
	item == Character lf
		ifTrue: [
			(position > 0 and: [ (buffer at: position) == Character cr ])
				ifTrue: [
					position := position - 1.
					^ nil ] ].
	self store: item.
	^ item
			
	! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 17:29'!
reset
	position := 0! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 18:43'!
store: item
	position >= buffer size ifTrue: [ self growBuffer ].
	position := position + 1.
	buffer at: position put: item		
	! !

Object subclass: #ZnLogListener
	instanceVariableNames: 'enabled filter lock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnLogListener commentStamp: 'SvenVanCaekenberghe 3/29/2011 11:47' prior: 0!
I am ZnLogListener.

I am an abstract class, subclasses should implment #event:.
I add filtering and monitor for exclusive access.

Part of Zinc HTTP Components.!

ZnLogListener subclass: #ZnFileLogger
	instanceVariableNames: 'stream pathName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnFileLogger commentStamp: 'SvenVanCaekenberghe 3/29/2011 09:37' prior: 0!
I am ZnFileLogger, a ZnLogListener that appends to a file.

Part of Zinc HTTP Components.!

!ZnFileLogger classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/29/2011 12:32'!
onFileNamed: pathName
	^ self new
		initializeOn: pathName;
		yourself  ! !

!ZnFileLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/20/2011 12:21'!
close
	self stream ifNil: [ ^ self ].
	self event: (ZnLogEvent info: 'Closing ', self printString).
	stream close. 
	stream := nil! !

!ZnFileLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:04'!
event: logEvent
	logEvent printOn: self stream.
	self stream lf! !

!ZnFileLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 12:34'!
initializeOn: aPathName 
	self pathName: aPathName.
	self open  ! !

!ZnFileLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 12:53'!
open
	| fileStream |
	self close.
	fileStream := FileStream fileNamed: self pathName.
	fileStream setToEnd.
	self stream: fileStream.
	self event: (ZnLogEvent info: 'Opening ', self printString) ! !

!ZnFileLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:33'!
pathName 
	^ pathName ! !

!ZnFileLogger methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 12:33'!
pathName: aPathName 
	pathName := aPathName  ! !

!ZnFileLogger methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:44'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self pathName.  
	aStream nextPut: $)! !

!ZnFileLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:03'!
stream
	^ stream! !

!ZnFileLogger methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 12:35'!
stream: aStream
	stream := aStream! !

!ZnLogListener methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 10:11'!
acceptEvent: logEvent
	filter isNil ifTrue: [ ^ true ].
	filter isSymbol ifTrue: [ ^ logEvent category = filter ].
	^ filter value: logEvent category  ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:30'!
allow: collection
	self filter: [ :category | collection includes: category ]! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:29'!
allowAll
	self filter: nil! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:38'!
enabled 
	^ enabled ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:38'!
enabled: boolean
	enabled := boolean! !

!ZnLogListener methodsFor: 'accessing' stamp: ''!
event: logEvent
	self subclassResponsibility ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:09'!
filter
	^ filter! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:09'!
filter: symbolOrBlock
	filter := symbolOrBlock ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:46'!
handleEvent: logEvent
	(self enabled and: [ self acceptEvent: logEvent ])
		ifTrue: [ 
			lock critical: [ self event: logEvent ] ]  ! !

!ZnLogListener methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 11:45'!
initialize 
	lock := Monitor new.
	self enabled: true  ! !

ZnLogListener subclass: #ZnMemoryLogger
	instanceVariableNames: 'logEvents limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnMemoryLogger commentStamp: 'SvenVanCaekenberghe 3/29/2011 09:52' prior: 0!
I am ZnMemoryLogger, a ZnLogListener that collects ZnLogEvents in memory.

Part of Zinc HTTP Components.!

!ZnMemoryLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 11:06'!
clear
	logEvents ifNotNil: [ logEvents removeAll ]  ! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:10'!
event: logEvent
	(self limit notNil and: [ self logEvents size >= self limit ])
		ifTrue: [ self shrink ].  
	self logEvents add: logEvent ! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:06'!
limit
	^ limit! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:06'!
limit: count
	limit := count! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:05'!
logEvents 
	^ logEvents ifNil: [ logEvents := OrderedCollection new ]! !

!ZnMemoryLogger methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 11:15'!
shrink
	(self limit / 10) rounded timesRepeat: [ self logEvents removeFirst ]  ! !

ZnLogListener subclass: #ZnTranscriptLogger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnTranscriptLogger commentStamp: 'SvenVanCaekenberghe 3/29/2011 09:38' prior: 0!
I am ZnTranscriptLogger, a ZnLogListener that appends to the Transcript.

Part of Zinc HTTP Components.!

!ZnTranscriptLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:37'!
event: logEvent
	Transcript cr; show: logEvent! !

Object subclass: #ZnLogSupport
	instanceVariableNames: 'announcer enabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnLogSupport commentStamp: 'SvenVanCaekenberghe 3/29/2011 11:39' prior: 0!
I am ZnLogSupport, I help HTTP servers and clients to manage ZnLogEvents.

To use the logging framework, instanciate your own log support: 

	log := ZnLogSupport new.

and use it to log messages:

	log info: 'Started server'.
	log debug: [ 'Created ', someComplexObject printString ].

To view the log, add a log listener:

	log addListener: ZnTranscriptLogger new.
	
Make sure to enable your log support as it is off by default:

	log enabled: true.

Part of Zinc HTTP Components.!

!ZnLogSupport classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/29/2011 11:40'!
newEnabled
	^ self new
		enabled: true;
		yourself  ! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 15:40'!
addListener: logListener
	self enabled: true.  
	self announcer on: ZnLogEvent send: #handleEvent: to: logListener ! !

!ZnLogSupport methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/28/2011 22:04'!
announcer
	^ announcer ifNil: [ announcer := Announcer new ]! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
debug: message
	self enabled 
		ifTrue: [  
			self announcer announce: (ZnLogEvent debug: message value) ]
	! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:29'!
enabled
	^ enabled ! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:30'!
enabled: boolean
	enabled := boolean ! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
info: message
	self enabled 
		ifTrue: [
			self announcer announce: (ZnLogEvent info: message value) ]
	! !

!ZnLogSupport methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 10:12'!
initialize 
	self enabled: false! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:18'!
removeListener: logListener
	self announcer unsubscribe: logListener ! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
transaction: message
	self enabled 
		ifTrue: [  
			self announcer announce: (ZnLogEvent transaction: message value) ]
	! !

Object subclass: #ZnMessage
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMessage commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:46' prior: 0!
I am ZnMessage, an abstract class representing an HTTP Message.

Part of Zinc HTTP Components.!

!ZnMessage classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMessage classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 12:18'!
readHeaderFrom: stream
	^ self new
		readHeaderFrom: stream;
		yourself! !

!ZnMessage classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readStreamingFrom: stream
	^ self new
		readStreamingFrom: stream;
		yourself! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:17'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:15'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/28/2010 13:03'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
entity
	^ entity! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2011 13:17'!
entity: object
	"Set my entity to object. Unless I already am describing an entity using my content type and length
	set my content type and length as defined by object."

	(entity := object) isNil ifTrue: [ ^ self ].
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMessage methodsFor: 'private' stamp: ''!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		yourself! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:02'!
hasEntity
	^ self entity notNil! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 20:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMessage methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:04'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:51'!
isConnectionClose
	| value |
	value := self headers at: 'Connection' ifAbsent: [ ^ false ].
	^ value sameAs: 'close'! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:51'!
isConnectionKeepAlive
	| value |
	value := self headers at: 'Connection' ifAbsent: [ ^ false ].
	^ value sameAs: 'keep-alive'! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/25/2010 22:43'!
readFrom: stream
	self readHeaderFrom: stream.
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: ''!
readHeaderFrom: stream
	self headers: (ZnHeaders readFrom: stream)
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 12:17'!
readStreamingFrom: stream
	| entityReader |
	self readHeaderFrom: stream.
	(entityReader := self entityReaderOn: stream)
		streaming.
	self entity: entityReader readEntity
	! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2011 13:21'!
resetEntity: object
	"Set my entity to object. Always set my content type and length as defined by object,
	even if I am already describing an entity using my content type and length."

	(entity := object) isNil ifTrue: [ ^ self ].
	self headers acceptEntityDescription: object! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 22:32'!
setConnectionClose
	self headers at: 'Connection' put: 'close'! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 13:58'!
setConnectionKeepAlive
	self headers at: 'Connection' put: 'keep-alive'! !

!ZnMessage methodsFor: 'writing' stamp: ''!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self headers writeOn: bivalentWriteStream.
	bivalentWriteStream nextPutAll: String crlf.
	self hasEntity ifTrue: [ self entity writeOn: bivalentWriteStream ]! !

ZnMessage subclass: #ZnRequest
	instanceVariableNames: 'requestLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequest commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:58' prior: 0!
I am ZnRequest, representing an HTTP Request, 
consisting of a request line, headers and an optional entity (body).
I am a ZnMessage.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequest classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
delete: urlObject
	^ self method: #DELETE url: urlObject! !

!ZnRequest classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:00'!
get: urlObject
	^ self method: #GET url: urlObject! !

!ZnRequest classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
head: urlObject
	^ self method: #HEAD url: urlObject! !

!ZnRequest classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 14:01'!
method: method url: urlObject
	| url |
	url := urlObject asZnUrl.
	^ self new
		requestLine: (ZnRequestLine method: method uri: url);
		headers: (ZnHeaders requestHeadersFor: url);
		yourself! !

!ZnRequest classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
post: urlObject
	^ self method: #POST url: urlObject! !

!ZnRequest classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
put: urlObject
	^ self method: #PUT url: urlObject! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
isHttp10
	^ self requestLine isHttp10 ! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
isHttp11
	^ self requestLine isHttp11! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:35'!
method
	^ self requestLine method ! !

!ZnRequest methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:00'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self requestLine printMethodAndUriOn: stream.
	stream nextPut: $)! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:33'!
readHeaderFrom: stream
	self requestLine: (ZnRequestLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine
	^ requestLine ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine: object
	requestLine := object! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 20:00'!
setAccept: object
	self headers at: 'Accept' put: object asString! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:28'!
setAcceptEncodingGzip
	self headers at: 'Accept-Encoding' put: 'gzip'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/21/2011 20:28'!
setBasicAuthenticationUsername: username password: password
	self headers 
		at: 'Authorization' 
		put: 'Basic ', (ZnUtils encodeBase64: (username, ':', password)) 
! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:36'!
uri
	^ self requestLine uri ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 21:48'!
url
	^ self uri ! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
wantsConnectionClose
	^ self isConnectionClose 
		or: [ self isHttp10 and: [ self isConnectionKeepAlive not ] ]! !

!ZnRequest methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:36'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self requestLine writeOn: bivalentWriteStream.
	super writeOn: bivalentWriteStream! !

ZnMessage subclass: #ZnResponse
	instanceVariableNames: 'statusLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnResponse commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:59' prior: 0!
I am ZnResponse, representing an HTTP Response 
consisting of a status line, headers and an optional entity (body).
I am a ZnMessage.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 13:48'!
badRequest: request
	^ self new
		statusLine: (ZnStatusLine code: 400);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: 'Bad Request ', request method, ' ', request uri pathQueryFragmentPrintString, String crlf);
		yourself! !

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 13:49'!
created: url
	| location |
	location := url asZnUrl pathQueryFragmentPrintString.
	^ self new
		statusLine: (ZnStatusLine code: 201);
		headers: ZnHeaders defaultResponseHeaders;
		setLocation: location;
		entity: (ZnEntity text: 'Created ', location, String crlf);
		yourself! !

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 13:49'!
notFound: url
	^ self new
		statusLine: (ZnStatusLine code: 404);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: 'Not Found ', url asZnUrl pathQueryFragmentPrintString, String crlf);
		yourself! !

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 16:38'!
ok: entity
	^ self new
		statusLine: (ZnStatusLine code: 200);
		headers: ZnHeaders defaultResponseHeaders;
		entity: entity;
		yourself! !

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 13:49'!
redirect: url
	| location |
	location := url asZnUrl pathQueryFragmentPrintString.
	^ self new
		statusLine: (ZnStatusLine code: 302);
		headers: ZnHeaders defaultResponseHeaders;
		setLocation: location;
		entity: (ZnEntity text: 'Redirect ', location, String crlf);
		yourself! !

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/4/2011 19:47'!
serverError: string
	^ self new
		statusLine: (ZnStatusLine code: 500);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: string, String crlf);
		yourself! !

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:46'!
unauthorized
	^ self unauthorized: 'Basic realm=ZincHTTPComponents'! !

!ZnResponse classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
unauthorized: authString
	^ self new
		statusLine: (ZnStatusLine code: 401);
		headers: ZnHeaders defaultResponseHeaders;
		setWWWAuthenticate: authString;
		entity: (ZnEntity text: 'Unauthorized', String crlf);
		yourself! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 20:44'!
code
	^ self statusLine code! !

!ZnResponse methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 19:02'!
entityReaderOn: stream
	"Reading undefined content is allowed for responses"
	
	^ (super entityReaderOn: stream)  
		allowReadingUpToEnd;
		yourself! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:03'!
isAuthenticationRequired
	^ self code = 401 ! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/21/2011 10:45'!
isNotModified
	^ self code = 304! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:03'!
isRedirect
	^ #(301 302) includes: self code! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/28/2010 12:38'!
isSuccess
	^ #(200 201 202 204) includes: self code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:31'!
location
	^ self headers at: 'Location'! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:27'!
location: uriString
	^ self headers at: 'Location'! !

!ZnResponse methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 12:54'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self statusLine printCodeAndReasonOn: stream.
	self hasEntity ifTrue: [ 
		stream space.
		self entity printContentTypeAndLengthOn: stream ].
	stream nextPut: $)
	! !

!ZnResponse methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:34'!
readHeaderFrom: stream
	self statusLine: (ZnStatusLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:34'!
setKeepAliveFor: request
	(request isHttp10 and: [ request isConnectionKeepAlive ])
		ifTrue: [ self setConnectionKeepAlive ]
	! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setLocation: uriString
	self headers at: 'Location' put: uriString! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setWWWAuthenticate: string
	self headers at: 'WWW-Authenticate' put: string! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 18:35'!
status
	^ self statusLine code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine
	^ statusLine! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine: object
	statusLine := object! !

!ZnResponse methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:36'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self statusLine writeOn: bivalentWriteStream.
	super writeOn: bivalentWriteStream! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:28'!
wwwAuthenticate
	^ self headers at: 'WWW-Authenticate'! !

Object subclass: #ZnMimePart
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMimePart commentStamp: 'SvenVanCaekenberghe 10/5/2010 19:20' prior: 0!
I am ZnMimePart.
I hold headers and an optional entity.
I am used in MultiPartMimeEncoding.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMimePart classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/31/2011 20:13'!
fieldName: fieldName fileName: fileName entity: entity
	^ self new
		setContentDisposition: 'form-data;name="', fieldName, '";filename="', fileName, '"';
		entity: entity;
		yourself! !

!ZnMimePart classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/31/2011 20:13'!
fieldName: fieldName value: fieldValue
	^ self new
		setContentDisposition: 'form-data;name="', fieldName, '"';
		entity: (ZnEntity bytes: fieldValue asByteArray);
		yourself! !

!ZnMimePart classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 19:57'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:12'!
contentDisposition
	^ self headers at: 'Content-Disposition' ifAbsent: [ nil ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/6/2010 09:30'!
contentDispositionValues
	| contentDisposition values start semiColonIndex |
	(contentDisposition := self contentDisposition) isNil ifTrue: [ ^ #() ].
	values := OrderedCollection new: 3.
	start := 1.
	semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ].
	[ semiColonIndex isZero ] whileFalse: [
		| value |
		value := contentDisposition copyFrom: start to: semiColonIndex - 1.
		values add: (ZnUtils trimString: value).
		start := semiColonIndex + 1.
		semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ] ].
	values add: (ZnUtils trimString: (contentDisposition copyFrom: start to: contentDisposition size)).
	^ values! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:01'!
detectContentDispositionValue: aString
	self contentDispositionValues do: [ :each | 
		((each beginsWith: aString)
			and: [ each size > (aString size + 1)
			and: [ (each at: aString size + 1) = $= ] ]) ifTrue: [
				^ (each copyAfter: $=) withoutQuoting ] ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
entity
	^ entity! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
entity: object
	(entity := object) isNil ifTrue: [ ^ self ].
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/3/2010 16:10'!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		allowReadingUpToEnd;
		yourself! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:03'!
fieldName
	^ self detectContentDispositionValue: 'name'! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:54'!
fieldValue
	^ self hasEntity 
		ifTrue: [ self entity contents ] 
		ifFalse: [ nil ] ! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:54'!
fieldValueString
	^ self fieldValue asString! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:01'!
fileName
	^ self detectContentDispositionValue: 'filename'! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
hasEntity
	^ self entity notNil! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 13:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMimePart methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMimePart methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/30/2010 19:54'!
readFrom: stream
	self headers: (ZnHeaders readFrom: stream).
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:49'!
setContentDisposition: value
	^ self headers at: 'Content-Disposition' put: value ! !

!ZnMimePart methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:54'!
writeOn: stream
	self headers writeOn: stream.
	stream nextPutAll: String crlf.
	self hasEntity ifTrue: [ self entity writeOn: stream ]! !

Object subclass: #ZnMimeType
	instanceVariableNames: 'main sub parameters'
	classVariableNames: 'ExtensionsMap MimeTypes'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnMimeType commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I am ZnMimeType, a copy from WAMineType.

I know whether I am binary or not and know my charset when applicable.

My class side offers access to some common constants.
Note that for text types we default to UTF-8 encoding.

Part of Zinc HTTP Components.!

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
applicationFormUrlEncoded
	^ MimeTypes at: #applicationFormUrlEncoded! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
applicationJavascript
	^ MimeTypes at: #applicationJavascript! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/25/2010 20:43'!
applicationJson
	^ MimeTypes at: #applicationJson! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
applicationOctetStream
	^ MimeTypes at: #applicationOctetStream! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/5/2011 12:17'!
applicationXml
	^ MimeTypes at: #applicationXml! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 15:48'!
default
	^ self applicationOctetStream ! !

!ZnMimeType classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/16/2010 20:29'!
forFilenameExtension: extension
	^ ExtensionsMap 
		at: extension 
		ifAbsent: [ self default ]! !

!ZnMimeType classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/14/2010 21:47'!
fromString: aString 
	| main endOfSub sub parts parameters |
	main := aString copyUpTo: $/.
	endOfSub := aString indexOf: $;.
	endOfSub := endOfSub = 0 
		ifTrue: [ aString size ]
		ifFalse: [ endOfSub - 1 ].
	sub := aString copyFrom: main size + 2 to: endOfSub.
	endOfSub = aString size ifTrue: [ ^ self main: main sub: sub ].
	parts := (aString copyFrom: endOfSub + 1 to: aString size) subStrings: ';'.
	parameters := Dictionary new.
	parts do: [ :each | 
		parameters
			at: (each copyUpTo: $=) withBlanksTrimmed 
			put: (each copyAfter: $=) withBlanksTrimmed ].
	^ self 
		main: main
		sub: sub
		parameters: parameters! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
imageGif
	^ MimeTypes at: #imageGif! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
imageJpeg
	^ MimeTypes at: #imageJpeg! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
imagePng
	^ MimeTypes at: #imagePng! !

!ZnMimeType classSide methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/16/2010 20:27'!
initialize
	self initializeMimeTypes.
	self initializeExtensionsMap! !

!ZnMimeType classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 12:16'!
initializeExtensionsMap 
	"To be extended later"
	
	ExtensionsMap := Dictionary new
		at: 'txt' put: self textPlain;
		at: 'text' put: self textPlain;
		at: 'htm' put: self textHtml;
		at: 'html' put: self textHtml;
		at: 'gif' put: self imageGif;
		at: 'jpg' put: self imageJpeg;
		at: 'jpeg' put: self imageJpeg;
		at: 'png' put: self imagePng;
		at: 'js' put: self applicationJavascript;
		at: 'xml' put: self applicationXml;
		at: 'css' put: self textCss;
		yourself! !

!ZnMimeType classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/25/2011 09:44'!
initializeMimeTypes 
	"To be extended later"

	MimeTypes := Dictionary new
		at: #applicationOctetStream put: (self main: 'application' sub: 'octet-stream');
		at: #applicationJavascript put: (self main: 'application' sub: 'javascript');
		at: #applicationJson put: (self main: 'application' sub: 'json');
		at: #applicationXml put: (self main: 'application' sub: 'xml');
		at: #applicationFormUrlEncoded put: (self main: 'application' sub: 'x-www-form-urlencoded');
		at: #imageGif put: (self main: 'image' sub: 'gif');
		at: #imageJpeg put: (self main: 'image' sub: 'jpeg');
		at: #imagePng put: (self main: 'image' sub: 'png');
		at: #textPlain put: ((self main: 'text' sub: 'plain') setCharSetUTF8; yourself);
		at: #textHtml put: ((self main: 'text' sub: 'html') setCharSetUTF8; yourself);
		at: #textCss put: (self main: 'text' sub: 'css');
		at: #textJavascript put: (self main: 'text' sub: 'javascript');
		at: #multiPartFormData put: (self main: 'multipart' sub: 'form-data');
		yourself! !

!ZnMimeType classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/7/2010 14:46'!
main: aMainString sub: aSubString
	^self new
		main: aMainString;
		sub: aSubString;
		yourself! !

!ZnMimeType classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/7/2010 14:46'!
main: aMainString sub: aSubString parameters: aDictionary
	^(self main: aMainString sub: aSubString)
		parameters: aDictionary;
		yourself! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
multiPartFormData
	^ MimeTypes at: #multiPartFormData! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
textCss
	^ MimeTypes at: #textCss! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
textHtml
	^ MimeTypes at: #textHtml! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/25/2011 09:45'!
textJavascript
	^ MimeTypes at: #textJavascript! !

!ZnMimeType classSide methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
textPlain
	^ MimeTypes at: #textPlain! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:35'!
= other
	^(self class == other class)
		and: [ self main = other main
		and: [ self sub = other sub ] ]! !

!ZnMimeType methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ self! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 10/5/2010 17:23'!
charSet
	^self parameterAt: 'charset' ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:24'!
charSet: aString
	self parameters at: 'charset' put: aString! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:48'!
hash
	^self main hash bitXor: self sub hash! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:45'!
isBinary
	"answers whether the contents of a document of the receiving mime type are binary"
	| subTypes |
	self main = 'text' ifTrue: [ ^false ].
	self = self class applicationJson ifTrue: [ ^ false ].
	self main = 'application' ifFalse: [ ^true ].
	subTypes := self sub subStrings: '+'.
	^ subTypes noneSatisfy: [ :each | 
		#('x-javascript' 'xml') includes: each ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/2/2010 11:19'!
isCharSetUTF8
	^ self charSet notNil and: [ self charSet sameAs: 'utf-8' ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:10'!
isNonStandard
	"tests if the receiver is a non-standard mime type that is not registered with IANA"
	^(self main beginsWith: 'x-')
		or: [ (self main beginsWith: 'X-')
		or: [ (self sub beginsWith: 'x-')
		or: [ self sub beginsWith: 'X-' ] ] ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:10'!
isVendorSpecific
	"tests if the receiver is a vendor specific mimetype"
	^self sub beginsWith: 'vnd.'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main
	^ main! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main: aString
	main := aString! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:51'!
matches: aMimeType
	"aMimeType is the pattern to match, it is a normal instance where main or sub can be wildcards"
	^(aMimeType main = '*'
		or: [ aMimeType main = self main ])
		and: [
			aMimeType sub = '*'
				or: [ aMimeType sub = self sub ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 10/5/2010 17:23'!
parameterAt: key
	^ self parameterAt: key ifAbsent: [ self error: 'Parameter not found: ', key asString ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 10/5/2010 19:30'!
parameterAt: key ifAbsent: block
	parameters isNil ifTrue: [ block value ].
	^ self parameters at: key ifAbsent: block! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 9/7/2010 14:50'!
parameterAt: aKey put: aValue
	self parameters at: aKey put: aValue! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:52'!
parameters
	parameters isNil ifTrue: [ parameters := Dictionary new ].
	^ parameters! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
parameters: aDictionary
	parameters := aDictionary! !

!ZnMimeType methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:49'!
printOn: stream
	stream
		nextPutAll: self main;
		nextPut: $/;
		nextPutAll: self sub.
	parameters isNil ifFalse: [ 
		parameters keysAndValuesDo: [ :key :value | 
			stream
				nextPut: $;;
				nextPutAll: key;
				nextPut: $=;
				nextPutAll: value ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:20'!
setCharSetUTF8
	self charSet: 'utf-8'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub
	^ sub! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub: aString
	sub := aString! !

Object subclass: #ZnMonticelloServerDelegate
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnMonticelloServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnMonticelloServerDelegate.
I am a proof of concept implementation of a basic Monticello server.
I function as a delegate for ZnServer.

ZnServer startDefaultOn: 1701.
ZnServer default delegate: ((ZnMonticelloServerDelegate new) 
									directory: (FileDirectory on: '/Users/sven/Tmp/monticello'); 
									yourself).

Part of Zinc HTTP Components.!

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:20'!
directory
	^ directory ! !

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:20'!
directory: fileDirectory
	directory := fileDirectory! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 14:49'!
handleGetMczEntry: request
	| name |
	name := request uri firstPathSegment.
	^ ((self isValidMczName: name) and: [ self directory fileExists: name ]) 
		ifTrue: [ | file entity |
			(file := self directory readOnlyFileNamed: name)
				binary.
			(entity := ZnStreamingEntity type: ZnMimeType applicationOctetStream length: file size)
				stream: file.
			ZnResponse ok: entity ]
		ifFalse: [ ZnResponse notFound: request uri ]! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
handleListMczEntries: request
	^ ZnResponse ok: (ZnEntity html: self repositoryListing )! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 14:49'!
handlePutMczEntry: request
	| name |
	name := request uri firstPathSegment.
	^ ((self isValidMczName: name) and: [ (self directory fileExists: name) not ]) 
		ifTrue: [
			self directory newFileNamed: name do: [ :file |
				file binary.
				request entity writeOn: file ].
			ZnResponse created: request uri ]
		ifFalse: [ ZnResponse badRequest: request ]! !

!ZnMonticelloServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 1/4/2011 13:42'!
handleRequest: request
	(request method = #GET)
		ifTrue: [
			^ request uri isSlash
				ifTrue: [ self handleListMczEntries: request ]
				ifFalse: [ self handleGetMczEntry: request ] ].
	^ (request method = #PUT) 
		ifTrue: [ self handlePutMczEntry: request ]
		ifFalse: [ ZnResponse badRequest: request ]! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 10:24'!
isValidMczName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcz')
				and: [ (string includes: $/) not ] ] ]! !

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:21'!
mczEntries
	^ (self directory fileNames select: [ :each | each endsWith: '.mcz' ])
		asSortedCollection! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 11:00'!
repositoryListing
	^ String streamContents: [ :str |
		str nextPutAll: '<html><head><title>Monticello Repository</title></head>'.
		str nextPutAll: '<body><h1>Monticello Repository</h1>'.
		str nextPutAll: '<ul>'.
		self mczEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>' ].
		str nextPutAll: '</ul></body></html>' ]! !

Object subclass: #ZnNetworkingUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnNetworkingUtils commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:03' prior: 0!
I am ZnNetworkingUtils holding various utilities related to HTTP networking.

Part of Zinc HTTP Components.!

!ZnNetworkingUtils classSide methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 1/7/2011 19:42'!
ipAddressToString: byteArray
	^ String streamContents: [ :stream |
			byteArray 
				do: [ :each | stream print: each ] 
				separatedBy: [ stream nextPut: $. ] ]! !

!ZnNetworkingUtils classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2011 19:42'!
isProxySet
	^ HTTPSocket httpProxyServer isEmpty not! !

!ZnNetworkingUtils classSide methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/24/2011 08:05'!
listenBacklogSize
	"Server socket backlog size (number of pending connection waiting to be accepted)"
	
	^ 32! !

!ZnNetworkingUtils classSide methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 3/24/2011 08:06'!
serverSocketOn: port
	| socket |
	socket := Socket newTCP.
	self setServerSocketOptions: socket.
	socket listenOn: port backlogSize: self listenBacklogSize.
	socket isValid
		ifFalse: [ self error: 'Cannot create socket on port ', port printString ].
	^ socket! !

!ZnNetworkingUtils classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/24/2011 08:06'!
setServerSocketOptions: socket
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: self socketBufferSize;
		setOption: 'SO_RCVBUF' value: self socketBufferSize ! !

!ZnNetworkingUtils classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/24/2011 08:07'!
setSocketStreamParameters: stream
	stream 
		binary;
		shouldSignal: true;
		autoFlush: false;
		bufferSize: self socketBufferSize;
		timeout: self socketStreamTimeout.
! !

!ZnNetworkingUtils classSide methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/24/2011 08:05'!
socketBufferSize
	"Size in bytes for Sockets and SocketStream IO buffers"

	^ 4096! !

!ZnNetworkingUtils classSide methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 3/24/2011 08:08'!
socketStreamOn: socket
	| stream |
	stream := SocketStream on: socket.
	self setSocketStreamParameters: stream.  
	^ stream! !

!ZnNetworkingUtils classSide methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 1/7/2011 19:41'!
socketStreamTimeout
	"Timeout in seconds for SocketStream IO"
	
	^ 10! !

!ZnNetworkingUtils classSide methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 3/24/2011 08:08'!
socketStreamToHostNamed: hostname port: port
	| stream |
	stream := SocketStream openConnectionToHostNamed: hostname port: port.
	self setSocketStreamParameters: stream.  
	^ stream
	! !

!ZnNetworkingUtils classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2011 19:42'!
socketStreamToProxy
	^ self 
		socketStreamToHostNamed: HTTPSocket httpProxyServer 
		port: HTTPSocket httpProxyPort ! !

!ZnNetworkingUtils classSide methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 1/7/2011 19:42'!
socketStreamToUrl: url
	| stream |
	stream := self isProxySet 
		ifTrue: [ self socketStreamToProxy ] 
		ifFalse: [ self socketStreamToUrlDirectly: url ].
	^ stream! !

!ZnNetworkingUtils classSide methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2011 19:42'!
socketStreamToUrlDirectly: url
	^ self socketStreamToHostNamed: url host port: url portOrDefault ! !

Object subclass: #ZnRequestLine
	instanceVariableNames: 'method uri version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequestLine commentStamp: '<historical>' prior: 0!
I am an HTTP Request Line.
I consist of an HTTP method, an URI and an HTTP protocol version.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequestLine classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
method: method uri: uri
	^ self new
		method: method;
		uri: uri;
		version: ZnConstants defaultHTTPVersion;
		yourself ! !

!ZnRequestLine classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 13:50'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnRequestLine methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:30'!
isHttp10
	^ self version = 'HTTP/1.0'! !

!ZnRequestLine methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:31'!
isHttp11
	^ self version = 'HTTP/1.1'! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
method
	^ method! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:36'!
method: string
	method := ZnConstants knownHTTPMethods 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknowHttpMethod method: string) signal ] ! !

!ZnRequestLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 13:56'!
printMethodAndUriOn: stream
	(self method notNil and: [self uri notNil])
		ifTrue: [
			stream nextPutAll: self method; space.
			self uri printPathQueryFragmentOn: stream ]! !

!ZnRequestLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:04'!
printOn: stream
	super printOn: stream.
	(self method notNil and: [self uri notNil])
		ifTrue: [
			stream nextPut: $(.
			self printMethodAndUriOn: stream.
			stream nextPut: $) ]! !

!ZnRequestLine methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 19:25'!
readFrom: stream
	| line lineStream |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self method: (lineStream upTo: Character space).
	self uri: (lineStream upTo: Character space).
	self version: (lineStream upToEnd)! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
uri
	^ uri! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:25'!
uri: anURI
	uri := anURI asZnUrl! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 13:56'!
uriPathQueryFragment
	^ String streamContents: [ :stream | 
			self uri printPathQueryFragmentOn: stream ]! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
version
	^ version! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:36'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknowHttpVersion version: string) signal ] ! !

!ZnRequestLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/4/2011 13:56'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self method; space.
	self uri printPathQueryFragmentOn: writeStream.
	writeStream space; nextPutAll: self version; nextPutAll: String crlf ! !

Object subclass: #ZnServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnServer commentStamp: 'SvenVanCaekenberghe 12/15/2010 15:38' prior: 0!
I am ZnServer, an abstract superclass of HTTP Servers.
I am a facade for controlling a default server implementation.

I delegate my public class protocol methods to #defaultServerClass.

ZnServer startDefaultOn: 1701.
ZnClient get: 'http://localhost:1701'.

Part of Zinc HTTP Components.!

!ZnServer classSide methodsFor: 'public' stamp: ''!
default
	"Return the Default instance that we manage"
	
	^ self defaultServerClass default ! !

!ZnServer classSide methodsFor: 'public' stamp: ''!
defaultOn: aNumber
	"Create a new Default instance on a given port,
	stopping any previously running Default and replacing it."
	
	^ self defaultServerClass defaultOn: aNumber  ! !

!ZnServer classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:54'!
defaultServerClass
	"Return the default ZnServer subclass to use"
	
	^ ZnMultiThreadedServer ! !

!ZnServer classSide methodsFor: 'class initialization' stamp: ''!
initialize
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self! !

!ZnServer classSide methodsFor: 'public' stamp: ''!
on: aNumber
	"Instantiate a new listener on a given port,
	send #start to it to start listening."

	^ self defaultServerClass on: aNumber ! !

!ZnServer classSide methodsFor: 'system startup' stamp: ''!
shutDown
	self subclassesDo: [ :each | each shutDown ]! !

!ZnServer classSide methodsFor: 'public' stamp: ''!
startDefaultOn: aNumber
	"Start the Default instance on a given port,
	stopping any previously running Default."

	^ self defaultServerClass startDefaultOn: aNumber ! !

!ZnServer classSide methodsFor: 'public' stamp: ''!
startOn: aNumber
	"Instantiate a new listener on a given port and start listening."

	^ self defaultServerClass startOn: aNumber  ! !

!ZnServer classSide methodsFor: 'system startup' stamp: ''!
startUp
	self subclassesDo: [ :each | each startUp ]! !

!ZnServer classSide methodsFor: 'public' stamp: ''!
stopDefault
	"Stop the Default instance, if any, and clear it."

	^ self defaultServerClass stopDefault ! !

!ZnServer methodsFor: 'testing' stamp: ''!
isListening
	"Return true when I have a valid server socket listening at the correct port"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'testing' stamp: ''!
isRunning
	"Return true when I am running"
	
	self subclassResponsibility  ! !

!ZnServer methodsFor: 'accessing' stamp: ''!
port
	"Return the integer port number we are (or will be) listening on"
	
	self subclassResponsibility  ! !

!ZnServer methodsFor: 'accessing' stamp: ''!
port: integer
	"Set the port number we will be listening on.
	Cannot be changed after the server is already running."
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'accessing' stamp: ''!
process
	"Return the process that is running my main listening loop.
	Will be nil when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'accessing' stamp: ''!
serverSocket
	"Return the server socket that I am using.
	Will be nil when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'public' stamp: ''!
start
	"Start me. I will start listening on my port for incoming HTTP connections.
	If I am running, I will first stop and thus effectively restart"
	
	self subclassResponsibility! !

!ZnServer methodsFor: 'public' stamp: ''!
stop
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	Does nothing when I am not running"
	
	self subclassResponsibility ! !

ZnServer subclass: #ZnSingleThreadedServer
	instanceVariableNames: 'port process serverSocket delegate authenticator log lastRequest lastResponse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

ZnSingleThreadedServer class
	instanceVariableNames: 'Default'!

!ZnSingleThreadedServer commentStamp: 'SvenVanCaekenberghe 12/15/2010 15:38' prior: 0!
I am ZnSingleThreadedServer.
I can be customized with a delegate (#handleRequest:) and an authenticator (#authenticateRequest:do:).

ZnSingleThreadedServer startDefaultOn: 1701.
ZnSingleThreadedServer default authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
ZnClient get: 'http://localhost:1701' username: 'foo' password: 'secret'.

I use ZnDefaultServerDelegate when no other delegate is set.
I am single threaded, I run in a single process.
I close connections after each request/response cycle.

Acknowledgement: basic design copied from Blackfoot.

Part of Zinc HTTP Components.!

ZnSingleThreadedServer class
	instanceVariableNames: 'Default'!

ZnSingleThreadedServer subclass: #ZnMultiThreadedServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnMultiThreadedServer commentStamp: 'SvenVanCaekenberghe 12/15/2010 15:39' prior: 0!
I am ZnMultiThreadedServer.
I inherit most features from ZnSingleThreadedServer.

ZnMultiThreadedServer startDefaultOn: 1701.
ZnMultiThreadedServer default authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
ZnClient get: 'http://localhost:1701' username: 'foo' password: 'secret'.

I am multi threaded, I fork a new process for each incoming request.
I try to keep connections alive in each process.

Part of Zinc HTTP Components.!

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 13:05'!
executeRequestResponseLoopOn: stream
	"Execute the HTTP request / response loop on stream in 3 steps
	#readRequest: #handleRequest: and #writeResponse:on:
	until the other end indicates it wants to stop or times out"
	
	| request response |
 	self log debug: 'Executing request/response loop'.
    [ (request := self readRequestSafely: stream) notNil
		ifTrue: [
			(response := self handleRequest: request) setKeepAliveFor: request.
	 		self writeResponse: response on: stream ].
	request isNil or: [ request wantsConnectionClose ] ] whileFalse ! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/10/2010 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid 
			ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionsOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 13:05'!
readRequestSafely: stream
	"Read request from stream, returning nil when the connection is closed or times out"
	
	^ [ self readRequest: stream ]
		on: ConnectionClosed, ConnectionTimedOut, ZnParseError 
		do: [ 
			self log debug: 'Connection closed while reading request'. 
			nil ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/30/2011 19:17'!
serveConnectionsOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeRequestResponseLoopOn: on it"
		 
	| stream socket |
	socket := (listeningSocket waitForAcceptFor: self acceptWaitTimeout) ifNil: [
		^ self noteAcceptWaitTimedOut ].
	stream := ZnNetworkingUtils socketStreamOn: socket.
	[ [ [ self executeRequestResponseLoopOn: stream ]
		ensure: [ self log debug: 'Closing stream'. stream close ] ]
			ifCurtailed: [ self log debug: 'Destroying socket'. socket destroy ] ]
				forkAt: Processor lowIOPriority
				named: self workerProcessName ! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/13/2010 16:33'!
workerProcessName
	^ String streamContents: [ :stream |
		stream nextPutAll: self class name; nextPutAll: ' HTTP worker' ]! !

!ZnSingleThreadedServer classSide methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:19'!
default
	"Return the Default instance that we manage"
	
	^ Default! !

!ZnSingleThreadedServer classSide methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:17'!
defaultOn: aNumber
	"Create a new Default instance on a given port,
	stopping any previously running Default and replacing it."

	Default isNil ifFalse: [ Default stop ].
	^ Default := self on: aNumber! !

!ZnSingleThreadedServer classSide methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 4/20/2011 12:12'!
initialize
	Default := nil! !

!ZnSingleThreadedServer classSide methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/7/2010 22:59'!
on: aNumber
	"Instantiate a new listener on a given port,
	send #start to it to start listening."

	^ self new port: aNumber; yourself! !

!ZnSingleThreadedServer classSide methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 12/7/2010 13:17'!
shutDown
	"Stop the Default instance, if any."
	
	Default isNil ifFalse: [ Default stop ]! !

!ZnSingleThreadedServer classSide methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:16'!
startDefaultOn: aNumber
	"Start the Default instance on a given port,
	stopping any previously running Default."

	^ (self defaultOn: aNumber) 
		start; 
		yourself! !

!ZnSingleThreadedServer classSide methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/7/2010 23:01'!
startOn: aNumber
	"Instantiate a new listener on a given port and start listening."

	^ (self on: aNumber) 
		start;
		yourself! !

!ZnSingleThreadedServer classSide methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 12/7/2010 13:19'!
startUp
	"Start the Default instance, if any."
	
	Default isNil ifFalse: [ Default start ]! !

!ZnSingleThreadedServer classSide methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:18'!
stopDefault
	"Stop the Default instance, if any, and clear it."
	
	self shutDown.
	Default := nil! !

!ZnSingleThreadedServer methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/30/2011 19:15'!
acceptWaitTimeout
	"How many seconds to we wait for a server socket listening for an accept ?"
	
	^ 300! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/10/2010 13:58'!
authenticateRequest: request do: block
	"Validate request and execute block.
	When our authenticator is not nil, pass the responsability"

	^ self authenticator isNil
		ifTrue: [ block value ]
		ifFalse: [ self authenticator authenticateRequest: request do: block ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:50'!
authenticator
	"Return the optional authenticator,
	the object that will be sent #authenticateRequest:do:
	to authenticate or refuse the requests. 
	When authentication succeeds, the block should be executed,
	when authentication fails, a appropriate response should be returned.
	If there is no authenticator, all requests will pass"
	
	^ authenticator ! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:48'!
authenticator: object
	"Set the object that will be sent #authenticateRequest:do:
	to authenticate or refuse the requests. Can be nil.
	When authentication succeeds, the block should be executed,
	when authentication fails, a appropriate response should be returned"

	authenticator := object! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 15:32'!
delegate
	"Return the optional delegate, 
	the object that will be sent #handleRequest: to handle a request and produce a response.
	The default delegate is ZnDefaultServerDelegate"
	
	delegate isNil ifTrue: [ delegate := ZnDefaultServerDelegate new ].
	^ delegate! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:48'!
delegate: object
	"Set the delegate to object. Can be nil.
	This will be sent #handleRequest: to handle a request and produce a response"
	
	delegate := object! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 13:05'!
executeOneRequestResponseOn: stream
	"Execute one single HTTP request / response cycle on stream in 3 steps:
	#readRequest: #handleRequest: and #writeResponse:on:
	Indicate that we intend to close the connection afterwards"

	| request response |
	self log debug: 'Executing 1 request/response cycle'.
	request := self readRequest: stream.
	(response := self handleRequest: request) setConnectionClose.
	self writeResponse: response on: stream! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 13:51'!
handleRequest: request
	"Handle request and return a response.
	If we have a delegate, pass the responsability.
	If we have no delegate, we return a page not found"

	| response initialMilliseconds |
	initialMilliseconds := Time millisecondClockValue.
	response := self authenticateRequest: request do: [
		self delegate isNil
			ifTrue: [ ZnResponse notFound: request uri ] 
			ifFalse: [ self delegate handleRequest: request ] ].
	self logRequest: request response: response started: initialMilliseconds.  
	^ response ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 13:04'!
initializeServerSocket
	"Set up a new server socket and listen on it"
	
	self log debug: 'Initializing server socket'.
	serverSocket := ZnNetworkingUtils serverSocketOn: port.! !

!ZnSingleThreadedServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/15/2010 21:24'!
isListening
	"Return true when I have a valid server socket listening at the correct port"
	
	^ self serverSocket notNil
		and: [ self serverSocket isValid and: [ self serverSocket localPort = self port ] ]! !

!ZnSingleThreadedServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:57'!
isRunning
	"Return true when I am running"
	
	^ self process notNil and: [ self serverSocket notNil ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:13'!
lastRequest
	"The last request that I received.
	Will only be tracked when logging is enabled"
	
	^ lastRequest ! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:13'!
lastResponse
	"The last response that I sent.
	Will only be tracked when logging is enabled"
	
	^ lastResponse ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/10/2010 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid
			 ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 13:02'!
log
	^ log ifNil: [ log := ZnLogSupport new ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 13:00'!
logRequest: request
	"Log a request that was just read"
	
	self log info: [ 
		lastRequest := request.
		'Read ', request printString ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 13:51'!
logRequest: request response: response started: initialMilliseconds
	self log transaction: [
		String streamContents: [ :stream |
			request requestLine printMethodAndUriOn: stream.  
			stream space; print: response code.
			response hasEntity ifTrue: [
				stream space; print: response entity contentLength; nextPut: $B ].  
			stream space; print: (Time millisecondsSince: initialMilliseconds); nextPutAll:  'ms' ] ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 13:00'!
logResponse: response
	"Log a response that was just written"
	
	self log info: [ 
		lastResponse := response.
		'Wrote ', response printString ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 15:40'!
logToTranscript
	"Setup logging to the Transcript"

	self log addListener: ZnTranscriptLogger new  ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/30/2011 19:15'!
noteAcceptWaitTimedOut
	"The accept wait on a server socket timed out (see #acceptWaitTimeout).
	This can be used as an opportunity to do some periodic tasks."
	
	self log debug: 'Wait for accept timed out'.
	self periodicTasks! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/30/2011 19:16'!
periodicTasks
	"Every #acceptWaitTimeout seconds this method is called.
	Note that during this time the server is not actively listening for connections."! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:51'!
port
	"Return the integer port number we are (or will be) listening on"
	
	^ port! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:53'!
port: integer
	"Set the port number we will be listening on.
	Cannot be changed after the server is already running."
	
	(self isRunning and: [ port ~= integer ]) 
		ifTrue: [ self error: 'Stop me before changing my port' ].
	port := integer! !

!ZnSingleThreadedServer methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 20:54'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream nextPutAll: (self isRunning ifTrue: [ 'running'] ifFalse: [ 'stopped' ]).
	self port notNil ifTrue: [ stream space; print: self port ].
	stream nextPut: $)! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:54'!
process
	"Return the process that is running my main listening loop.
	Will be nil when I am not running"
	
	^ process! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2011 19:50'!
readRequest: stream
	"Read a request from stream.
	We add a virtual header containing the remote IP address of the client"

	| request |
	request := ZnRequest readFrom: stream.
	request headers 
		at: ZnConstants remoteAddressHeader
		put: (ZnNetworkingUtils ipAddressToString: stream socket remoteAddress).
	self logRequest: request.
	^ request! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/20/2011 12:11'!
releaseServerSocket
	"Release our server socket"
	
	serverSocket ifNil: [ ^ self ].
	self log debug: 'Releasing server socket'.
	(Delay forMilliseconds: 10) wait.
	serverSocket destroy.
	serverSocket := nil! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/30/2011 19:16'!
serveConnectionOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeOneRequestResponseOn: on it"
		 
	| stream socket |
	socket := (listeningSocket waitForAcceptFor: self acceptWaitTimeout) ifNil: [
		^ self noteAcceptWaitTimedOut ].
	stream := ZnNetworkingUtils socketStreamOn: socket.
	[ [ [ self executeOneRequestResponseOn: stream ]
		ensure: [ self log debug: 'Closing stream'. stream close ] ]
			ifCurtailed: [ self log debug: 'Destroying socket'. socket destroy ] ]
				forkAt: Processor highIOPriority
				named: 'Zinc HTTP Server Worker'! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/13/2010 16:42'!
serverProcessName
	^ String streamContents: [ :stream |
		stream nextPutAll: self class name; nextPutAll: ' HTTP port '; print: self port ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:54'!
serverSocket
	"Return the server socket that I am using.
	Will be nil when I am not running"
	
	^ serverSocket! !

!ZnSingleThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 3/29/2011 13:05'!
start
	"Start me. I will start listening on my port for incoming HTTP connections.
	If I am running, I will first stop and thus effectively restart"
	
	self stop.
	self log info: [ 'Starting ', self serverProcessName ].
	process := [ [ self listenLoop ] repeat ]
		forkAt: Processor highIOPriority 
		named: self serverProcessName ! !

!ZnSingleThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 4/20/2011 12:11'!
stop
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	Does nothing when I am not running"
	
	self isRunning ifFalse: [ ^ self ].
	process terminate. 
	process := nil.
	self log info: [ 'Stopped ', self serverProcessName ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 13:03'!
stopLogging
	"Stop doing any logging by disabling our log support"

	self log enable: false! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/14/2010 14:37'!
writeResponse: response on: stream
	"Write response to stream and flush the stream"
	
	response writeOn: stream.
	stream flush.
	self logResponse: response ! !

Object subclass: #ZnStaticFileServerDelegate
	instanceVariableNames: 'prefix directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnStaticFileServerDelegate commentStamp: 'SvenVanCaekenberghe 4/26/2011 12:48' prior: 0!
I am ZnStaticFileServerDelegate.
I am a simple proof of concept implementation of a web server serving static files.
I handle urls with an optional prefix as requests for files in a directory.
I serve index.html or index.htm when a directory is requested and these files exist.
I do a redirect when a path that is not does not end with a / refers to directory.
I function as a delegate for ZnServer.

ZnServer startDefaultOn: 1701.
ZnServer default delegate: ((ZnStaticFileServerDelegate new) 
									prefixFromString: 'apple/macosx'; 
									directory: (FileDirectory on: '/Library/WebServer/Documents'); 
									yourself).

Part of Zinc HTTP Components.!

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 11:36'!
actualFilenameFor: uri
	| subElements subDir entry |
	(uri isEmpty and: [ self prefix isEmpty ]) ifTrue: [ ^ self indexFileIn: self directory ].
	(self prefix isEmpty or: [ uri pathSegments beginsWith: self prefix ]) ifFalse: [ ^ nil ].
	subElements := (uri pathSegments allButFirst: self prefix size) reject: [ :each | each = #/ ].
	subDir := subElements allButLast inject: self directory into: [ :parent :sub |
		(parent directoryExists: sub)
			ifTrue: [ parent directoryNamed: sub ]
			ifFalse: [ ^ nil ] ]. 
	subElements isEmpty
		ifTrue: [ entry := subDir containingDirectory directoryEntryFor: subDir localName ] 
		ifFalse: [ (entry := subDir directoryEntryFor: subElements last) ifNil: [ ^ nil ] ].
	^ entry isDirectory 
		ifTrue: [ self indexFileIn: entry asFileDirectory ]
		ifFalse: [ entry fullName ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:06'!
directory
	"The FileDirectory whose files I am serving"
	
	^ directory ! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:07'!
directory: fileDirectory
	"Set the FileDirectory whose files I should be serving"

	directory := fileDirectory! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 12:35'!
directoryRedirectFor: uri
	| directoryUri |
	directoryUri := uri copy addPathSegment: #/.  
	^ ZnResponse redirect: directoryUri ! !

!ZnStaticFileServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 4/26/2011 12:30'!
handleRequest: request
	"Server delegate entry point"
	
	| actualFilename |
	actualFilename := self actualFilenameFor: request uri.
	^ actualFilename notNil
		ifTrue: [ 
			(self redirectNeededFor: request uri actualFilename: actualFilename)
				ifTrue: [ self directoryRedirectFor: request uri ]    
				ifFalse: [ self responseForFile: actualFilename ] ] 
		ifFalse: [ ZnResponse notFound: request uri ]! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 13:16'!
indexFileIn: fileDirectory
	#( 'index.html' 'index.htm' ) do: [ :each |
			(fileDirectory fileExists: each)
				ifTrue: [ 
					^ (fileDirectory directoryEntryFor: each ) fullName ] ].
	^ nil
! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 12:46'!
prefix
	"The path prefix collection under which I am serving files"
	
	^ prefix ifNil: [ prefix := #() ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:08'!
prefix: orderedCollection
	"Set the path prefix under which I should be serving files to orderedCollection containing path elements"
	
	prefix := orderedCollection! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:09'!
prefixFromString: string
	"Set the path prefix under which I should be serving files to string,
	interpreting each /-separated token as a path element"

	self prefix: (string findTokens: '/')! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 13:11'!
redirectNeededFor: uri actualFilename: actualFilename
	uri isDirectory ifTrue: [ ^ false ].  
	^ (actualFilename endsWith: uri lastPathSegment) not! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/16/2010 16:34'!
responseForFile: filename
	| entry size time mimeType fileStream entity response |
	entry := FileDirectory root directoryEntryFor: filename.
	size := entry fileSize.
	time := entry modificationDateAndTime.
	mimeType := ZnMimeType forFilenameExtension: (FileDirectory extensionFor: filename).
	fileStream := FileDirectory root oldFileNamed: filename.
	mimeType isBinary ifTrue: [ fileStream binary ].
	(entity := ZnStreamingEntity type: mimeType length: size)
		stream: fileStream.
	(response := ZnResponse ok: entity)
		headers at: 'Modification-Date' put: (ZnUtils httpDate: time).
	^ response ! !

Object subclass: #ZnStatusLine
	instanceVariableNames: 'version code reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStatusLine commentStamp: '<historical>' prior: 0!
I am an HTTP Status Line.
I consist of an HTTP protocol version, an integer HTTP status code and an HTTP reason string.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnStatusLine classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:48'!
code: code
	^ self new
		code: code;
		version: ZnConstants defaultHTTPVersion;
		yourself! !

!ZnStatusLine classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:47'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:49'!
code
	^ code! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:35'!
code: integer
	(ZnConstants httpStatusCodes includesKey: integer) 
		ifFalse: [ 
			(ZnUnknowHttpStatusCode code: integer) signal ].
	code := integer! !

!ZnStatusLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 10:57'!
printCodeAndReasonOn: stream
	self code notNil
		ifTrue: [
			stream print: self code; space; nextPutAll: self reason ]! !

!ZnStatusLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 10:57'!
printOn: stream
	super printOn: stream.
	self code notNil
		ifTrue: [
			stream nextPut: $(. 
			self printCodeAndReasonOn: stream.
			stream nextPut: $) ]! !

!ZnStatusLine methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 2/28/2011 14:44'!
readFrom: stream
	| line lineStream httpCode |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self version: (lineStream upTo: Character space).
	httpCode := lineStream upTo: Character space.
	httpCode := Integer readFrom: httpCode ifFail: [ (ZnUnknowHttpStatusCode code: code) signal ].
	self code: httpCode.
	self reason: (lineStream upToEnd)
! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:27'!
reason
	reason isNil 
		ifTrue: [ 
			reason := ZnConstants httpStatusCodes at: self code ifAbsent: [ 'Unknown Status Code' ] ].
	^ reason! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 16:29'!
reason: string
	reason := string! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:46'!
version
	^ version! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:35'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknowHttpVersion version: version) signal ] ! !

!ZnStatusLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:40'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self version; space.
	self code printOn: writeStream.
	writeStream space; nextPutAll: self reason; nextPutAll: String crlf ! !

Object subclass: #ZnUrl
	instanceVariableNames: 'scheme host port segments query fragment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUrl commentStamp: 'SvenVanCaekenberghe 1/4/2011 15:11' prior: 0!
I am ZnUrl, an implementation of an interpreted HTTP(S) URL/URI. 
Contrary to other implementations, this one is specific for HTTP(S).

I consist of the following parts:
  - scheme - either #http, #https or nil
  - host - hostname string or nil
  - port - port integer or nil
  - segments - collection of path segments, ends with #/ for directories
  - query - query dictionary or nil
  - fragment - fragment string or nil

The methods in accessing protocols do not do any encoding, those in printing do.

Part of Zinc HTTP Components.!

!ZnUrl classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/3/2011 19:35'!
fromString: string
	^ (self new)
		parseFrom: string; 
		yourself ! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:00'!
= anObject
	^ self class == anObject class 
		and: [ self equals: anObject ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:22'!
addPathSegment: segment
	segments isNil ifTrue: [ 
		segments := OrderedCollection new ].
	segments add: segment! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 11:24'!
asZnUrl
	^ self! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 22:22'!
authority
	^ self hasPort
		ifTrue: [ 
			String streamContents: [ :stream | 
				stream nextPutAll: self host; nextPut: $:; print: self port ] ]
		ifFalse: [ 
			self host ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:59'!
authorityPrintString
	^ String streamContents: [ :stream | 
			self printAuthorityOn: stream ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 11:56'!
closePath
	self isSlash
		ifFalse: [ self addPathSegment: #/ ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:21'!
directory
	^ self isDirectory
		ifTrue: [ self path ]
		ifFalse: [ self joinSegments: segments allButLast ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/3/2011 16:17'!
encode: string on: stream
	stream nextPutAll: string encodeForHTTP 
! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:59'!
equals: url
	self scheme = url scheme ifFalse: [ ^ false ].
	self host = url host ifFalse: [ ^ false ].
	self portOrDefault = url portOrDefault ifFalse: [ ^ false ].
	((self isSlash and: [ url isSlash ]) or: [ self segments = url segments ]) ifFalse: [ ^ false ].
	self query = url query ifFalse: [ ^ false ].
	self fragment = url fragment ifFalse: [ ^ false ].
	^ true! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:15'!
file
	^ self isDirectory
		ifTrue: [ false ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:46'!
firstPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments first ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment
	^ fragment! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment: string
	fragment := string! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasFragment
	^ fragment notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasHost
	^ host notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:03'!
hasPath
	^ segments notNil and: [ segments isEmpty not ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasPort
	^ port notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:47'!
hasQuery
	^ query notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasScheme
	^ scheme notNil! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:43'!
hash
	^ self hasPath
		ifTrue: [ segments inject: host hash into: [ :sum :each | sum bitXor: each hash ] ]
		ifFalse: [ host hash ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
host
	^ host! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:14'!
host: hostname
	host := hostname! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:55'!
isAbsolute
	^ self hasScheme! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:57'!
isDirectory
	^ self hasPath 
		ifTrue: [ segments last = #/ ]
		ifFalse: [ true ]
	! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 20:06'!
isEmpty
	^ (self hasScheme | self hasHost | self hasPath | self hasQuery | self hasFragment) not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:56'!
isFile
	^ self isDirectory not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttp
	^ scheme == #http! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttps
	^ scheme == #https! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:55'!
isRelative
	^ self isAbsolute not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:54'!
isSlash
	^ self hasPath not or: [ segments size = 1 and: [ segments first = #/ ] ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/3/2011 22:20'!
joinSegments: collection
	^ String streamContents: [ :stream |
		collection 
			do: [ :each | stream nextPutAll: each]
			separatedBy: [ stream nextPut: $/ ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:51'!
lastPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:53'!
parseAuthority: stream
	self host: (ZnUtils decodePercent: (stream upTo: $:)).
	stream atEnd 
		ifFalse: [ 
			self port: (Integer readFrom: stream ifFail: [ ZnPortNotANumber signal ]) ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:53'!
parseFrom: string
	| start end index |
	start := 1.
	end := string size.
	(index := string indexOf: $#) > 0
		ifTrue: [ 
			self fragment: (ZnUtils decodePercent: (string copyFrom: index + 1 to: end)).
			end := index - 1].
	(index := string indexOf: $?) > 0
		ifTrue: [
			self query: (ZnUtils parseQueryFrom: (ReadStream on: string from: index + 1 to: end)).
			end := index - 1 ].
	(index := string indexOfSubCollection: '://') > 0
		ifTrue: [
			self scheme: (string copyFrom: 1 to: index - 1).
			start := index + 3.
			(index := string indexOf: $/ startingAt: start) > 0
				ifTrue: [
					self parseAuthority: (ReadStream on: string from: start to: index - 1).
					start := index ]
				ifFalse: [ 
					^ self parseAuthority: (ReadStream on: string from: start to: end) ] ].
	self parsePath: (ReadStream on: string from: start to: end)! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:53'!
parsePath: stream
	stream peekFor: $/.
	[ stream atEnd ] whileFalse: [ | segment |
		segment := String streamContents: [ :stringStream |
			[ stream atEnd not and: [ stream peek ~= $/ ] ] whileTrue: [
				stringStream nextPut: stream next ] ].
		(segment isEmpty or: [ segment = '.' ])
			ifFalse: [
				segment = '..' 
					ifTrue: [ self removeLastPathSegment ]
					ifFalse: [ self addPathSegment: (ZnUtils decodePercent: segment) ] ].
		((stream peekFor: $/) and: [ stream atEnd ])
			ifTrue: [ self closePath ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:21'!
path
	^ self joinSegments: segments! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:16'!
pathPrintString
	^ String streamContents: [ :stream | 
			self printPathOn: stream ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:17'!
pathQueryFragmentPrintString
	^ String streamContents: [ :stream | 
			self printPathQueryFragmentOn: stream ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:51'!
pathSegments
	^ segments! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
port
	^ port! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:14'!
port: integer
	port := integer! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:52'!
portOrDefault
	^ self hasPort
		ifTrue: [ self port ]
		ifFalse: [ 
			self hasScheme
				ifTrue: [ self isHttp ifTrue: [ 80 ] ifFalse: [ 443 ] ]
				ifFalse: [ nil ] ]! !

!ZnUrl methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 4/26/2011 12:34'!
postCopy
	super postCopy.
	segments := segments copy.
	query := query copy! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:58'!
printAuthorityOn: stream
	self hasHost ifTrue: [ 
		stream nextPut: $/; nextPut: $/. 
		self encode: self host on: stream ].
	self hasPort ifTrue: [ 
		stream nextPut: $:; print: self port ]
! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 15:10'!
printOn: stream
	self hasScheme ifTrue: [ 
		stream nextPutAll: self scheme; nextPut: $: ].
	self printAuthorityOn: stream.
	self printPathQueryFragmentOn: stream! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/17/2011 10:27'!
printPathOn: stream
	segments isNil ifTrue: [ ^ stream nextPut: $/ ].
 	segments do: [ :each | 
		stream nextPut: $/.
		each == #/
			ifFalse: [ 
				self encode: each on: stream ] ] ! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 14:38'!
printPathQueryFragmentOn: stream
	self printPathOn: stream.
	self hasQuery ifTrue: [ 
		self printQueryOn: stream ].
	self hasFragment ifTrue: [ 
		stream nextPut: $#. 
		self encode: self fragment on: stream ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:22'!
printQueryOn: stream
	stream nextPut: $?. 
	ZnUtils writeQueryFields: self query on: 	stream! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 16:26'!
query
	^ query! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 20:15'!
query: dictionary
	query := dictionary! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/12/2011 11:26'!
queryAddAll: keyedCollection
	"Note that we use #at:add:"
	
	keyedCollection keysAndValuesDo: [ :key :value |
		self queryAt: key add: value ].
	^ keyedCollection ! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/3/2011 19:35'!
queryAt: key
	^ self hasQuery
		ifTrue: [ self query at: key ]
		ifFalse: [ self error: 'key not found: ', key printString ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/12/2011 11:21'!
queryAt: key add: value
	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key add: value! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/3/2011 19:35'!
queryAt: key ifAbsent: block
	^ self hasQuery
		ifTrue: [ self query at: key ifAbsent: block ]
		ifFalse: [ block value ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/12/2011 11:21'!
queryAt: key put: value
	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key put: value! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/31/2011 12:18'!
queryDo: block
	self hasQuery ifTrue: [ 
		self query keysAndValuesDo: block ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/4/2011 12:00'!
queryRemoveKey: key
	self hasQuery
		ifTrue: [ self query removeKey: key ifAbsent: [] ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 11:55'!
removeLastPathSegment
	segments isEmpty 
		ifFalse: [ segments removeLast ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
scheme
	^ scheme! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:18'!
scheme: symbol
	symbol isNil 
		ifTrue: [ scheme := nil ]
		ifFalse: [ 
			(#(http https) includes: symbol) 
				ifTrue: [ scheme := symbol asSymbol ]
				ifFalse: [ (ZnUnknowScheme scheme: symbol) signal ] ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 10:56'!
segments
	^ segments! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 13:22'!
segments: collection
	segments := collection! !

Object subclass: #ZnUserAgent
	instanceVariableNames: 'session settings request response log errorHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnUserAgent commentStamp: '<historical>' prior: 0!
I implement a client for making HTTP requests. I can maintain session state across multiple requests, automatically follow redirects and notify observers of my request and response activity. The contents of my request headers can be modified easily.

Much of my behavior can be controlled with an instance of ZnUserAgentSettings.

Some examples of my use:

client := ZnUserAgent new.
response := client get: 'pharo-project.org'.
args := (Dictionary new) at: 'foo' put: 'bar'; yourself.
response := client post: 'mytestdomain.com/form' data: args.!

ZnUserAgent subclass: #ZnHttpClient
	instanceVariableNames: 'url params headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnHttpClient commentStamp: '<historical>' prior: 0!
I implement an HTTP client with a high level API.

Some examples of my use:

response := ZnHttpClient new
	url: 'somedomain.com';
	auth: 'joe' password: 'secret';
	parameterAt: 'name' put: 'joe';
	headerAt: 'X-Custom Header' put: 'woot';
	get.
	 !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:36'!
auth: username password: password
	| cred |
	cred := ZnBasicCredential new
				username: username;
				password: password;
				domain: self url authority;
				yourself.
	self session addCredential: cred! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:14'!
contentLength
	^self response contentLength! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:14'!
contentType
	^self response contentType! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 21:20'!
cookieAt: aName
	^(super cookieAt: aName) value! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/12/2011 13:29'!
delete
	self 
		sendMethod: [ self method: #DELETE for: self url headers: self headers data: self params ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:36'!
digestAuth: username password: password
	| cred |
	cred := ZnDigestCredential new
				username: username;
				password: password;
				domain: self url authority;
				yourself.
	self session addCredential: cred! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/20/2011 20:36'!
get
	self url queryAddAll: self params.
	self 
		sendMethod: [ self method: #GET for: self url headers: self headers data: nil ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/20/2011 20:36'!
head
	self url queryAddAll: self params.
	self 
		sendMethod: [ self method: #HEAD for: self url headers: self headers data: nil ];
		resetParameters.
	^ self response! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:27'!
headerAt: aName add: aValue
	self headers at: aName add: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:17'!
headerAt: aName put: aValue
	self headers at: aName put: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:27'!
headers
	^ headers ifNil: [ headers := ZnMultiValueDictionary new ]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
headers: anObject
	headers := anObject! !

!ZnHttpClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:08'!
isAuthenticationRequired
	^self response isAuthenticationRequired ! !

!ZnHttpClient methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 21:55'!
linkCredentials
	(self session credentialsForUrl: self url) do: [:cred |
		cred linkToAuthRequest: (self response headers at: 'WWW-Authenticate')]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:13'!
parameterAt: aName
	^self params at: aName ifAbsent: [nil]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:28'!
parameterAt: aName add: aValue
	self params at: aName add: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:15'!
parameterAt: aName put: aValue
	self params at: aName put: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:27'!
params
	^ params ifNil: [ params := ZnMultiValueDictionary new ]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
params: anObject
	params := anObject! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/12/2011 13:30'!
post
	self 
		sendMethod: [ self method: #POST for: self url headers: self headers data: self params ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/12/2011 13:30'!
put
	self 
		sendMethod: [ self method: #PUT for: self url headers: self headers data: self params ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/12/2011 10:16'!
resetParameters
	params ifNotNil: [ params removeAll ]! !

!ZnHttpClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/12/2011 10:08'!
sendMethod: aBlock
	aBlock on: Error do: self errorHandler.
	self isAuthenticationRequired ifTrue: [
		self linkCredentials.
		aBlock on: Error do: self errorHandler].
	self resetParameters ! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:14'!
status
	^self response code! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
url
	^ url! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 14:19'!
url: anObject
	url := anObject asZnUrl! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:12'!
userAgent
	^self settings agent! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:12'!
userAgent: aString
	self settings agent: aString! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:09'!
cookieAt: aName
	"Answers the named cookie for the current request or nil if non exists."
	
	^self session cookiejar cookieAt: aName forUrl: self request url! !

!ZnUserAgent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:47'!
defaultErrorHandler
	^ [ :error | 
			self log debug: [ error description ]. 
			error defaultAction ]! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 15:02'!
delete: urlObject
	^self delete: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 15:02'!
delete: urlObject headers: headerCollection
	^self method: #DELETE for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 13:57'!
errorHandler
	^errorHandler ifNil: [errorHandler := self defaultErrorHandler]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 13:45'!
errorHandler: aBlock
	errorHandler := aBlock! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:06'!
get: urlObject
	^self get: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:06'!
get: urlObject headers: headerCollection
	^self method: #GET for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:11'!
head: urlObject
	^self head: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:10'!
head: urlObject headers: headerCollection
	^self method: #HEAD for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:41'!
log
	^ log ifNil: [ log := ZnLogSupport new ]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:41'!
logProgressToTranscript
	self logToTranscript! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 15:40'!
logToTranscript
	self log addListener: ZnTranscriptLogger new    ! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/14/2011 20:27'!
method: aMethod for: urlObject headers: headerCollection data: dataEntity
	| limit |
	limit := self settings followRedirect ifTrue: [ self settings redirectLimit ] ifFalse: [ 0 ].
	^self method: aMethod for: urlObject headers: headerCollection data: dataEntity limit: limit.
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/20/2011 21:10'!
method: method for: url headers: headerCollection data: dataEntity limit: limit
	| stream redirects continue |
	redirects := 0.
	continue := true.
	self prepareRequest: url method: method headers: headerCollection data: dataEntity.
	[ [ continue ] whileTrue: [
			stream := self openConnection.
			request setConnectionClose.
			[ self sendRequest: stream ] ensure: [ stream close ].
			(self response isRedirect and: [ redirects < limit ])
				ifTrue: [ self prepareRedirect: headerCollection. redirects := redirects + 1 ]
				ifFalse: [ continue := false ] ] ]
		on: Error
		do: self errorHandler.
	^ self response! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/20/2011 19:36'!
openConnection
	| socketStream |
	socketStream := ZnNetworkingUtils socketStreamToUrl: request url.
	self settings timeout 
		ifNotNil: [ :timeout | socketStream timeout: timeout ].
	^ socketStream! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:07'!
post: urlObject data: dataEntity
	^self post: urlObject headers: nil data: dataEntity! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:07'!
post: urlObject headers: headerCollection data: dataEntity
	^self method: #POST for: urlObject headers: headerCollection data: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 10/6/2010 17:20'!
prepareCredentials: headers for: urlObject method: method
	(self session credentialsForUrl: urlObject) do: [:cred |
		cred isComplete ifTrue: [
			[cred setAuthorizationHeader: headers for: urlObject method: method]
				on: Error
				do: self errorHandler]]
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/12/2011 10:12'!
prepareData: dataEntity
	"Prepare the ZnEntity for the request."
	
	dataEntity ifNil: [ ^ dataEntity ].
	(dataEntity isKindOf: ZnEntity) ifTrue: [ ^ dataEntity ].
	^ ZnApplicationFormUrlEncodedEntity withAll: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 10/1/2010 21:35'!
prepareHeaders: headerCollection for: urlObject method: method
	| headers |
	headers := (ZnHeaders requestHeadersFor: urlObject)
					at: 'User-Agent' put: self settings agent;
					yourself.
	headerCollection isNil 
		ifFalse: [ headers addAll: headerCollection ].
	(self session cookiejar cookiesForUrl: urlObject) do: [:cookie |
		headers at: 'Cookie' add: cookie asString].
	self prepareCredentials: headers for: urlObject method: method.
	^headers
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/14/2011 21:04'!
prepareRedirect: headerCollection
	"We implement the common behavior that POST/PUT redirects turn into GET request.
	Note also that the body data is dropped"
	
	| method |
	(#(POST PUT) includes: request method)
		ifTrue: [ method := #GET ]
		ifFalse: [ method := self request method ].
	self prepareRequest: self redirectUrl method: method headers: headerCollection! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/26/2010 16:36'!
prepareRequest: urlObject method: method 
	^self prepareRequest: urlObject method:  method headers: nil! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/26/2010 16:36'!
prepareRequest: urlObject method: method headers: headerCollection
	^self prepareRequest: urlObject method:  method headers: headerCollection data: nil! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/4/2011 14:21'!
prepareRequest: urlObject method: method headers: headerCollection data: dataEntity
	| url |
	url := urlObject asZnUrl.
	request := ZnRequest new
		requestLine: (ZnRequestLine method: method uri: url);
		headers: (self prepareHeaders: headerCollection for: url method: method);
		entity: (self prepareData: dataEntity);
		yourself! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 3/29/2011 14:46'!
processResponse: stream
	| cookies |
	response := (request method = #HEAD) 
		ifTrue: [ZnResponse readHeaderFrom: stream]
		ifFalse: [ZnResponse readFrom: stream].
	self log debug: [ response printString ].
	cookies := response headers at: 'Set-Cookie' ifAbsent: [^response].
	cookies := (cookies isCollection & cookies isString) ifTrue: [Array with: cookies] ifFalse: [cookies].
	cookies do: [:cookie | 
		self session cookiejar add: (ZnCookie fromString: cookie for: self request url)].
	^response ! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:10'!
put: urlObject data: dataEntity
	^self put: urlObject headers: nil data: dataEntity! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:10'!
put: urlObject headers: headerCollection data: dataEntity
	^self method: #PUT for: urlObject headers: headerCollection data: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/20/2011 21:12'!
redirectUrl
	"We support relative locations although they are not the standard (http://en.wikipedia.org/wiki/HTTP_location)"
	
	| redirectUrl |
	redirectUrl := self response location asZnUrl.
	redirectUrl isAbsolute
		ifFalse: [ | baseUrl |
			baseUrl := self request url.
			redirectUrl 
				scheme: baseUrl scheme;
				host: baseUrl host;
				port: baseUrl port ].
	^ redirectUrl
! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
request
	^ request! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:09'!
request: znRequest
	request := znRequest! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
response
	^ response! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:09'!
response: znResponse
	response := znResponse! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 3/29/2011 14:46'!
sendRequest: stream
	self log debug: [ request printString ].
	request writeOn: stream.
	stream flush.
	^ self processResponse: stream! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 11:17'!
session
	^ session ifNil: [ session := self settings loadSession ]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:10'!
session: userAgentSession
	session := userAgentSession! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/26/2010 16:14'!
settings
	^ settings isNil 
		ifTrue: [settings := ZnUserAgentSettings defaultSettingsClass new]
		ifFalse: [settings]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:10'!
settings: userAgentSettings
	settings := userAgentSettings! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:47'!
url
	^ self request url! !

Object subclass: #ZnUserAgentSession
	instanceVariableNames: 'cookiejar credentials currentRealm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUserAgentSession commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I represent a session store for an HTTP client instance of ZnUserAgent. 

I hold the cookie jar for the session and any authentication credentials used by the client.

Part of Zinc HTTP Components.!

!ZnUserAgentSession classSide methodsFor: 'instance creation' stamp: 'MattKennedy 9/28/2010 10:52'!
withCookieJar: aCookieJar
	^self new cookiejar: aCookieJar; yourself! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:31'!
addCredential: aCredential
	self credentials at: aCredential domain, '@', aCredential realm put: aCredential ! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 00:00'!
cookiejar
	^ cookiejar ifNil: [ cookiejar := ZnCookieJar new ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
cookiejar: anObject
	cookiejar := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:32'!
credentialAt: aRealm forUrl: urlObject
	^self credentials at: urlObject authority, '@', aRealm ifAbsent: [nil]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:30'!
credentials
	^ credentials ifNil: [credentials := Dictionary new]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
credentials: anObject
	credentials := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 10/6/2010 17:19'!
credentialsForUrl: urlObject
	^self credentials values select: [:cred | urlObject authority includesSubString: (cred domain)]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm
	^ currentRealm! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm: anObject
	currentRealm := anObject! !

Object subclass: #ZnUserAgentSettings
	instanceVariableNames: 'settings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUserAgentSettings commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I manage the settings for a ZnUserAgent instance. 

I can be subclassed to extend capabilities to include loading settings from a configuration store or maintaining persistent storage for a cookie jar.

Part of Zinc HTTP Components.!

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:53'!
commentString
	^'(', self platform platformName, '; ', self encryptionStrengthComment, '; ', self platformDetails, '; ', self localeComment, ')'! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 00:00'!
defaultCookieJar
	^ ZnCookieJar ! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:54'!
defaultSessionClass
	^ZnUserAgentSession! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 21:00'!
defaultSettingsClass
	^ self ! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/1/2010 21:43'!
defaultUserAgentString
	^ZnConstants frameworkNameAndVersion, ' ', self commentString, ' ',  self platform version asString! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:51'!
encryptionStrengthComment
	"For user agent comment. Standard values are U, I, or N. Currently returning N until standardize https transport is available."
	^'N'! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:49'!
localeComment
	^LocaleID current isoString! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/6/2010 09:32'!
platform
	^ SmalltalkImage current os! !

!ZnUserAgentSettings classSide methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:51'!
platformDetails
	^self platform platformName, ' ', self platform platformSubtype, ' ', self platform osVersion! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:31'!
agent
	^ settings at: #agent ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:38'!
agent: aString
	settings at: #agent put: aString ! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:32'!
followRedirect
	^ settings at: #followRedirect ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:39'!
followRedirect: aBoolean
	settings at: #followRedirect put: aBoolean! !

!ZnUserAgentSettings methodsFor: 'initialize-release' stamp: 'MattKennedy 9/26/2010 16:12'!
initialize
	super initialize.
	self settings at: #agent put: self class defaultUserAgentString.
	self followRedirect: true.
	self redirectLimit: 7.
	self timeout: 180! !

!ZnUserAgentSettings methodsFor: 'operations' stamp: 'MattKennedy 9/28/2010 10:55'!
loadSession
	^self class defaultSessionClass withCookieJar: self class defaultCookieJar new! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:33'!
redirectLimit
	^ settings at: #redirectLimit ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:39'!
redirectLimit: anInteger
	settings at: #redirectLimit put: anInteger! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:31'!
settings
	^ settings isNil ifTrue: [ settings := Dictionary new] ifFalse: [settings]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:30'!
settings: anObject
	settings := anObject! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:33'!
timeout
	^ settings at: #timeout ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:39'!
timeout: anInteger
	settings at: #timeout put: anInteger! !

Object subclass: #ZnUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUtils commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:03' prior: 0!
I am ZnUtils holding various utilities related to HTTP handling.

Part of Zinc HTTP Components.!

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/10/2010 20:14'!
capitalizeString: string
	| capitalExpected |
	capitalExpected := true.
	^ String new: string size streamContents: [ :stream |
		string do: [ :each |
			each isAlphaNumeric
				ifTrue: [
					capitalExpected 
						ifTrue: [ 
							stream nextPut: each asUppercase.
							capitalExpected := false. ]
						ifFalse: [ 
							stream nextPut: each asLowercase ] ] 
		 		ifFalse: [ 
					stream nextPut: each.
					capitalExpected := true ] ] ]! !

!ZnUtils classSide methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 13:10'!
currentProcessID
	"Return a integer of at most 6 digits that can be used as a description for the current active process.
	This is a reduced hash that is not necessarily unique (but uniqueness is very probably)"
	
	^ Processor activeProcess hash \\ 999999! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 3/21/2011 20:11'!
decodeBase64: string
	^ string base64Decoded ! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/28/2011 15:51'!
decodePercent: string
	^ [ string unescapePercents ]
		on: Error 
		do: [ (ZnPercentEncodingWrong string: string) signal ]  ! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 4/26/2011 18:58'!
encodeBase64: string
	^ (Base64MimeConverter respondsTo: #mimeEncode:multiLine:)
		ifTrue: [ 
			(Base64MimeConverter mimeEncode: string readStream multiLine: false) contents ]	
		ifFalse: [ 
			(Base64MimeConverter mimeEncode: string readStream) contents 
				reject: [ :each | each = Character cr ] ]! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/16/2010 15:52'!
httpDate
	^ self httpDate: DateAndTime now! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/17/2010 14:49'!
httpDate: dateAndTime
	"Answers a printer that formats dates for HTTP1.1 (RFC 1123). Eg. Sun, 06 Nov 1994 08:49:37 GMT"

	| utc |
	utc := dateAndTime asUTC.
	^ String streamContents: [ :stream |
		stream nextPutAll: utc dayOfWeekAbbreviation; nextPut: $,; space.
		stream nextPutAll: (utc dayOfMonth printStringLength: 2 padded: true).
		stream space; nextPutAll: utc monthAbbreviation; space.
		utc year printOn: stream.
		stream space.
		utc printHMSOn: stream.
		stream nextPutAll: ' GMT' ]! !

!ZnUtils classSide methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:00'!
isCapitalizedString: string
	| capitalExpected |
	capitalExpected := true.
	string do: [ :each |
		each isAlphaNumeric
			ifTrue: [
				((capitalExpected and: [ each isUppercase])
					or: [ capitalExpected not and: [ each isLowercase] ])
					ifTrue: [ capitalExpected := false ]
					ifFalse: [ ^ false ] ] 
		 	ifFalse: [ capitalExpected := true ] ].
	^ true



! !

!ZnUtils classSide methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/27/2010 15:00'!
nextPutAll: collection on: stream chunked: size
	| written |
	written := 0.
	[ written < collection size ] whileTrue: [ | chunkSize |
		chunkSize := size min: (collection size - written).
		chunkSize printOn: stream base: 16.
		stream nextPutAll: String crlf.
		chunkSize timesRepeat: [ stream nextPut: (collection at: (written := written + 1)) ].
		stream nextPutAll: String crlf].
	stream nextPutAll: String crlf! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/28/2011 15:52'!
parseQueryFrom: stream
	| string index fields |
	fields := ZnMultiValueDictionary new.
	[ stream atEnd ] whileFalse: [
		string := stream upTo: $&.
		index := string indexOf: $=.
		index > 0
			ifFalse: [ fields at: (self decodePercent: string) put: nil ]
			ifTrue: [
				fields
					at: (self decodePercent: (string copyFrom: 1 to: index - 1))
					add: (self decodePercent: (string copyFrom: index + 1 to: string size)) ] ].
	^ fields! !

!ZnUtils classSide methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/17/2010 13:40'!
streamFrom: inputStream to: outputStream size: totalSize
	| bufferSize buffer leftToRead |
	bufferSize := 4096 min: totalSize.
	buffer := (inputStream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: bufferSize.
	leftToRead := totalSize.
	[ leftToRead > 0 ]
		whileTrue: [ | readCount |
			readCount := bufferSize min: leftToRead.
			inputStream next: readCount into: buffer.
			leftToRead := leftToRead - readCount.
			outputStream next: readCount putAll: buffer startingAt: 1 ]! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 10/5/2010 21:24'!
trimString: string
	"Until String>>#trimBoth is available everywhere"
	
	| left right |
	left := 1.
	right := string size.
	[ left <= right and: [ (string at: left) isSeparator ] ]
		whileTrue: [ left := left + 1 ].
	[ left <= right and: [ (string at: right) isSeparator ] ]
		whileTrue: [ right := right - 1 ].
	^ string copyFrom: left to: right! !

!ZnUtils classSide methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/13/2010 21:32'!
writeQueryFields: dictionary on: stream
	| first |
	first := true.
	dictionary keysAndValuesDo: [ :key :value |
		first ifFalse: [ stream nextPut: $& ].
		first := false.
		stream nextPutAll: key encodeForHTTP.
		value isNil ifFalse: [
			stream 
				nextPut: $=;
				nextPutAll: value encodeForHTTP ] ]! !

Object subclass: #ZnValueDelegate
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnValueDelegate commentStamp: 'SvenVanCaekenberghe 12/7/2010 16:14' prior: 0!
I am ZnValueDelegate.
I forward #handleRequest: messages to the object that I wrap using #value:

ZnServer default delegate: 
	(ZnValueDelegate with: [ :request | 
		ZnResponse ok: (ZnEntity with: 'You asked for ', request uri printString) ] ).

Part of Zinc HTTP Components.!

!ZnValueDelegate classSide methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/7/2010 16:15'!
with: object
	^ (super basicNew) object: object; yourself! !

!ZnValueDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 16:16'!
handleRequest: request
	^ object value: request! !

!ZnValueDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 16:15'!
object: anObject
	object := anObject! !
ZnByteEncoder initialize!
ZnConstants initialize!
ZnMimeType initialize!
ZnServer initialize!
ZnSingleThreadedServer initialize!
